@@ -1,5 +1,5 @@
 /* Printing of backtraces and error messages
- * Copyright (C) 1996 Mikael Djurfeldt
+ * Copyright (C) 1996,1997 Free Software Foundation
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -99,14 +99,16 @@ scm_display_error_message (message, args, port)
   char *start;
   char *p;
   
-  if (!SCM_STRINGP (message) || SCM_IMP (args) || !scm_list_p (args))
+  if (SCM_IMP (message) || !SCM_ROSTRINGP (message) || SCM_IMP (args)
+      || !scm_list_p (args))
     {
       scm_prin1 (message, port, 0);
       scm_gen_putc ('\n', port);
       return;
     }
 
-  start = SCM_CHARS (message);
+  SCM_COERCE_SUBSTR (message);
+  start = SCM_ROCHARS (message);
   for (p = start; *p != '\0'; ++p)
     if (*p == '%')
       {
@@ -231,8 +233,6 @@ display_frame_expr (hdr, exp, tlr, indentation, sport, port, pstate)
      SCM port;
      scm_print_state *pstate;
 {
-  pstate->level = 2;
-  pstate->length = 3;
   if (SCM_NIMP (exp) && SCM_CONSP (exp))
     {
       scm_iprlist (hdr, exp, tlr[0], port, pstate);
@@ -243,6 +243,57 @@ display_frame_expr (hdr, exp, tlr, indentation, sport, port, pstate)
   scm_gen_putc ('\n', port);
 }
 
+static void display_application SCM_P ((SCM frame, int indentation, SCM sport, SCM port, scm_print_state *pstate));
+static void
+display_application (frame, indentation, sport, port, pstate)
+     SCM frame;
+     int indentation;
+     SCM sport;
+     SCM port;
+     scm_print_state *pstate;
+{
+  SCM proc = SCM_FRAME_PROC (frame);
+  SCM name = (SCM_NFALSEP (scm_procedure_p (proc))
+	      ? scm_procedure_name (proc)
+	      : SCM_BOOL_F);
+  display_frame_expr ("[",
+		      scm_cons (SCM_NFALSEP (name) ? name : proc,
+				SCM_FRAME_ARGS (frame)),
+		      SCM_FRAME_EVAL_ARGS_P (frame) ? " ..." : "]",
+		      indentation,
+		      sport,
+		      port,
+		      pstate);
+}
+
+SCM_PROC(s_display_application, "display-application", 1, 1, 0, scm_display_application);
+
+SCM
+scm_display_application (SCM frame, SCM port)
+{
+  if (SCM_UNBNDP (port))
+    port = scm_cur_outp;
+  if (SCM_FRAME_PROC_P (frame))
+    /* Display an application. */
+    {
+      SCM print_state;
+      scm_print_state *pstate;
+      
+      /* Create a print state for printing of frames. */
+      print_state = scm_make_print_state ();
+      pstate = SCM_PRINT_STATE (print_state);
+      pstate->writingp = 1;
+      pstate->fancyp = 1;
+      pstate->level = 2;
+      pstate->length = 9;
+      
+      display_application (frame, 0, SCM_BOOL_F, port, pstate); /*fixme*/
+      return SCM_BOOL_T;
+    }
+  else
+    return SCM_BOOL_F;
+}
+
 static void display_frame SCM_P ((SCM frame, int nfield, int indentation, SCM sport, SCM port, scm_print_state *pstate));
 static void
 display_frame (frame, nfield, indentation, sport, port, pstate)
@@ -280,20 +331,7 @@ display_frame (frame, nfield, indentation, sport, port, pstate)
 
   if (SCM_FRAME_PROC_P (frame))
     /* Display an application. */
-    {
-      SCM proc = SCM_FRAME_PROC (frame);
-      SCM name = (SCM_NFALSEP (scm_procedure_p (proc))
-		  ? scm_procedure_name (proc)
-		  : SCM_BOOL_F);
-      display_frame_expr ("[",
-			  scm_cons (SCM_NFALSEP (name) ? name : proc,
-				    SCM_FRAME_ARGS (frame)),
-			  SCM_FRAME_EVAL_ARGS_P (frame) ? " ..." : "]",
-			  nfield + 1 + indentation,
-			  sport,
-			  port,
-			  pstate);
-    }
+    display_application (frame, nfield + 1 + indentation, sport, port, pstate);
   else
     /* Display a special form. */
     {
@@ -382,6 +420,8 @@ scm_display_backtrace (stack, port, first, depth)
   pstate = SCM_PRINT_STATE (print_state);
   pstate->writingp = 1;
   pstate->fancyp = 1;
+  pstate->level = 2;
+  pstate->length = 3;
 
   /* First find out if it's reasonable to do indentation. */
   if (SCM_BACKWARDS_P)
@@ -427,10 +467,46 @@ scm_display_backtrace (stack, port, first, depth)
   return SCM_UNSPECIFIED;
 }
 
+SCM_VCELL (scm_has_shown_backtrace_hint_p_var, "has-shown-backtrace-hint?");
+
+SCM_PROC(s_backtrace, "backtrace", 0, 0, 0, scm_backtrace);
+SCM
+scm_backtrace ()
+{
+  if (SCM_NFALSEP (SCM_CDR (scm_the_last_stack_var)))
+    {
+      scm_newline (scm_cur_outp);
+      scm_display_backtrace (SCM_CDR (scm_the_last_stack_var),
+			     scm_cur_outp,
+			     SCM_UNDEFINED,
+			     SCM_UNDEFINED);
+      scm_newline (scm_cur_outp);
+      if (SCM_FALSEP (SCM_CDR (scm_has_shown_backtrace_hint_p_var))
+	  && !SCM_BACKTRACE_P)
+	{
+	  scm_gen_puts (scm_regular_string,
+			"Type \"(debug-enable 'backtrace)\" if you would like "
+			"a backtrace\n"
+			"automatically if an error occurs in the future.\n",
+			scm_cur_outp);
+	  SCM_SETCDR (scm_has_shown_backtrace_hint_p_var, SCM_BOOL_T);
+	}
+    }
+  else
+    {
+      scm_gen_puts (scm_regular_string,
+		    "No backtrace available.\n",
+		    scm_cur_outp);
+    }
+  return SCM_UNSPECIFIED;
+}
+
 
 
 void
 scm_init_backtrace ()
 {
+  scm_the_last_stack_var = scm_sysintern ("the-last-stack", SCM_BOOL_F);
+
 #include "backtrace.x"
 }
