@@ -2,7 +2,7 @@
 
 #ifndef BOOLEANH
 #define BOOLEANH
-/*	Copyright (C) 1995 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@
  * If you do not wish that, delete this exception notice.  
  */
 
-#include "__scm.h"
+#include "libguile/__scm.h"
 
 
 
@@ -60,17 +60,8 @@
 
 
 
-#ifdef __STDC__
-extern SCM scm_not(SCM x);
-extern SCM scm_boolean_p(SCM obj);
-extern void scm_init_boolean (void);
-
-#else /* STDC */
-extern SCM scm_not();
-extern SCM scm_boolean_p();
-extern void scm_init_boolean ();
-
-#endif /* STDC */
-
+extern SCM scm_not SCM_P ((SCM x));
+extern SCM scm_boolean_p SCM_P ((SCM obj));
+extern void scm_init_boolean SCM_P ((void));
 
 #endif  /* BOOLEANH */
