@@ -1,5 +1,5 @@
 /* Debugging extensions for Guile
- * Copyright (C) 1995, 1996 Mikael Djurfeldt
+ * Copyright (C) 1995, 1996 Free Software Foundation
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -116,7 +116,7 @@ scm_single_step (cont, val)
   SCM_ENTER_FRAME_P = SCM_EXIT_FRAME_P = 1;
   SCM_RESET_DEBUG_MODE;
   SCM_ALLOW_INTS;
-  scm_throw (cont, val);
+  scm_call_continuation (cont, val);
   return SCM_BOOL_F; /* never returns */
 }
 
@@ -385,6 +385,8 @@ scm_make_debugobj (frame)
 
 
 
+/* Undocumented debugging procedure */
+#ifdef GUILE_DEBUG
 SCM_PROC (s_debug_hang, "debug-hang", 0, 1, 0, scm_debug_hang);
 
 SCM
@@ -395,6 +397,7 @@ scm_debug_hang (obj)
   while (!go) ;
   return SCM_UNSPECIFIED;
 }
+#endif
 
 
 
