@@ -2,7 +2,7 @@
 
 #ifndef DEBUGH
 #define DEBUGH
-/*	Copyright (C) 1995,1996 Mikael Djurfeldt
+/*	Copyright (C) 1995,1996 Free Software Foundation
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -100,9 +100,9 @@ extern int scm_check_entry_p, scm_check_apply_p, scm_check_exit_p;
 
 #define SCM_RESET_DEBUG_MODE \
 {\
-  if (SCM_ENTER_FRAME_P || SCM_BREAKPOINTS_P) CHECK_ENTRY = 1;\
-  if (SCM_APPLY_FRAME_P || SCM_TRACE_P) CHECK_APPLY = 1;\
-  if (SCM_EXIT_FRAME_P || SCM_TRACE_P) CHECK_EXIT = 1;\
+  CHECK_ENTRY = SCM_ENTER_FRAME_P || SCM_BREAKPOINTS_P;\
+  CHECK_APPLY = SCM_APPLY_FRAME_P || SCM_TRACE_P;\
+  CHECK_EXIT = SCM_EXIT_FRAME_P || SCM_TRACE_P;\
   scm_debug_mode = SCM_DEVAL_P || CHECK_ENTRY || CHECK_APPLY || CHECK_EXIT;\
   scm_ceval_ptr = scm_debug_mode ? scm_deval : scm_ceval;\
 }
