@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -58,73 +58,11 @@
  */
 
 
-/* True between SCM_DEFER_INTS and SCM_ALLOW_INTS, and
- * when the interpreter is not running at all.
- */
-int scm_ints_disabled = 1;
-
 extern int errno;
 
-static void err_head SCM_P ((char *str));
-
-static void 
-err_head (str)
-     char *str;
-{
-  int oerrno = errno;
-  if (SCM_NIMP (scm_cur_outp))
-    scm_fflush (scm_cur_outp);
-  scm_gen_putc ('\n', scm_cur_errp);
-#if 0
-  if (SCM_BOOL_F != *scm_loc_loadpath)
-    {
-      scm_prin1 (*scm_loc_loadpath, scm_cur_errp, 1);
-      scm_gen_puts (scm_regular_string, ", line ", scm_cur_errp);
-      scm_intprint ((long) scm_linum, 10, scm_cur_errp);
-      scm_gen_puts (scm_regular_string, ": ", scm_cur_errp);
-    }
-#endif
-  scm_fflush (scm_cur_errp);
-  errno = oerrno;
-  if (scm_cur_errp == scm_def_errp)
-    {
-      if (errno > 0)
-	perror (str);
-      fflush (stderr);
-      return;
-    }
-}
-
-
-SCM_PROC(s_errno, "errno", 0, 1, 0, scm_errno);
-SCM
-scm_errno (arg)
-     SCM arg;
-{
-  int old = errno;
-  if (!SCM_UNBNDP (arg))
-    {
-      if (SCM_FALSEP (arg))
-	errno = 0;
-      else
-	errno = SCM_INUM (arg);
-    }
-  return SCM_MAKINUM (old);
-}
-
-SCM_PROC(s_perror, "perror", 1, 0, 0, scm_perror);
-SCM 
-scm_perror (arg)
-     SCM arg;
-{
-  SCM_ASSERT (SCM_NIMP (arg) && SCM_STRINGP (arg), arg, SCM_ARG1, s_perror);
-  err_head (SCM_CHARS (arg));
-  return SCM_UNSPECIFIED;
-}
-
 void (*scm_error_callback) () = 0;
 
-/* all errors thrown from C should pass through here.  */
+/* All errors should pass through here.  */
 void
 scm_error (key, subr, message, args, rest)
      SCM key;
@@ -146,10 +84,38 @@ scm_error (key, subr, message, args, rest)
   
   /* No return, but just in case: */
 
-  write (2, "unhandled system error", sizeof ("unhandled system error") - 1);
+  write (2, "unhandled system error\n",
+	 sizeof ("unhandled system error\n") - 1);
   exit (1);
 }
 
+/* Scheme interface to scm_error.  */
+SCM_PROC(s_error_scm, "scm-error", 5, 0, 0, scm_error_scm);
+SCM
+scm_error_scm (key, subr, message, args, rest)
+     SCM key;
+     SCM subr;
+     SCM message;
+     SCM args;
+     SCM rest;
+{
+  SCM_ASSERT (SCM_NIMP (key) && SCM_SYMBOLP (key), key, SCM_ARG1, s_error_scm);
+  SCM_ASSERT (SCM_FALSEP (subr) || (SCM_NIMP (subr) && SCM_ROSTRINGP (subr)),
+	      subr, SCM_ARG2, s_error_scm);
+  SCM_ASSERT (SCM_FALSEP (message)
+	      || (SCM_NIMP (message) && SCM_ROSTRINGP (message)),
+	      message, SCM_ARG3, s_error_scm);
+
+  SCM_COERCE_SUBSTR (message);
+
+  scm_error (key,
+	     (SCM_FALSEP (subr)) ? NULL : SCM_ROCHARS (subr),
+	     (SCM_FALSEP (message)) ? NULL : SCM_ROCHARS (message),
+	     args,
+	     rest);
+  /* not reached.  */
+}
+
 SCM_SYMBOL (scm_system_error_key, "system-error");
 void
 scm_syserror (subr)
@@ -164,16 +130,17 @@ scm_syserror (subr)
 }
 
 void
-scm_syserror_msg (subr, message, args)
+scm_syserror_msg (subr, message, args, eno)
      char *subr;
      char *message;
      SCM args;
+     int eno;
 {
   scm_error (scm_system_error_key,
 	     subr,
 	     message,
 	     args,
-	     scm_listify (SCM_MAKINUM (errno), SCM_UNDEFINED));
+	     scm_listify (SCM_MAKINUM (eno), SCM_UNDEFINED));
 }
 
 void
@@ -323,6 +290,7 @@ scm_wta (arg, pos, s_subr)
 void
 scm_init_error ()
 {
+#include "cpp_err_symbols.c"
 #include "error.x"
 }
 
