@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -54,6 +54,23 @@
 
 #ifndef DEVAL
 
+/* AIX requires this to be the first thing in the file.  The #pragma
+   directive is indented so pre-ANSI compilers will ignore it, rather
+   than choke on it.  */
+#ifndef __GNUC__
+# if HAVE_ALLOCA_H
+#  include <alloca.h>
+# else
+#  ifdef _AIX
+ #pragma alloca
+#  else
+#   ifndef alloca /* predefined by HP cc +Olibcalls */
+char *alloca ();
+#   endif
+#  endif
+# endif
+#endif
+
 #include <stdio.h>
 #include "_scm.h"
 #include "debug.h"
@@ -657,7 +674,6 @@ iqq (form, env, depth)
       --depth;
     label:
       form = SCM_CDR (form);
-      /* !!! might need a check here to be sure that form isn't a struct. */
       SCM_ASSERT (SCM_NIMP (form) && SCM_ECONSP (form) && SCM_NULLP (SCM_CDR (form)),
 	      form, SCM_ARG1, s_quasiquote);
       if (0 == depth)
@@ -737,7 +753,7 @@ scm_m_define (x, env)
     {
       x = evalcar (x, env);
 #ifdef DEBUG_EXTENSIONS
-      if (SCM_REC_PROCNAMES_P)
+      if (SCM_REC_PROCNAMES_P && SCM_NIMP (x) && SCM_CLOSUREP (x))
 	scm_set_procedure_property_x (x, scm_i_name, proc);
 #endif
       arg1 = scm_sym2vcell (proc, env_top_level (env), SCM_BOOL_T);
@@ -899,11 +915,11 @@ scm_m_cont (xorig, env)
  * environment and rewrites it to its original form.  Thus, it is the
  * inversion of the rewrite rules above.  The procedure is not
  * optimized for speed.  It's used in scm_iprin1 when printing the
- * code of a closure, in scm_procedure_source and in scm_expr_stack
- * when generating the source for a stackframe.
+ * code of a closure, in scm_procedure_source, in display_frame when
+ * generating the source for a stackframe in a backtrace, and in
+ * display_expression.
  */
 
-
 static SCM unmemocopy SCM_P ((SCM x, SCM env));
 
 static SCM
@@ -1061,6 +1077,7 @@ unmemocopy (x, env)
 	  ls = z = scm_cons (scm_i_atcall_cc, SCM_UNSPECIFIED);
 	  goto loop;
 	default:
+	  /* appease the Sun compiler god: */ ;
 	}
     unmemo:
     default:
@@ -1138,6 +1155,40 @@ scm_eval_args (l, env)
     }
   return res;
 }
+
+
+/* The SCM_CEVAL and SCM_APPLY functions use this function instead of
+   calling setjmp directly, to make sure that local variables don't
+   have their values clobbered by a longjmp.
+
+   According to Harbison & Steele, "Automatic variables local to the
+   function containing setjmp are guaranteed to have their correct
+   value in ANSI C only if they have a volatile-qualified type or if
+   their values were not changed between the original call to setjmp
+   and the corresponding longjmp call."
+
+   SCM_CEVAL and SCM_APPLY are too complex for me to see how to meet
+   the second condition, and making x and env volatile would be a
+   speed problem, so we'll just trivially meet the first, by having no
+   "automatic variables local to the function containing setjmp."  */
+
+/* Actually, this entire approach is bogus, because setjmp ends up
+   capturing the stack frame of the wrapper function, which then
+   returns, rendering the jump buffer invalid.  Duh.  Gotta find a
+   better way...  -JimB  */
+#define safe_setjmp(x) setjmp (x)
+static int
+unsafe_setjmp (jmp_buf env)
+{
+  /* I think ANSI requires us to write the function this way, instead
+     of just saying "return setjmp (env)".  Maybe I'm being silly.
+     See Harbison & Steele, third edition, p. 353.  */
+  int val;
+  val = setjmp (env);
+  return val;
+}
+
+
 #endif /* !DEVAL */
 
 
@@ -1175,7 +1226,10 @@ scm_eval_args (l, env)
     if (SCM_APPLY_FRAME_P || (SCM_TRACE_P && PROCTRACEP (proc)))\
       {\
 	SCM tmp, tail = SCM_TRACED_FRAME_P (debug) ? SCM_BOOL_T : SCM_BOOL_F;\
-	SCM_SET_TRACED_FRAME (debug);\
+	SCM_SET_TRACED_FRAME (debug); \
+	SCM_APPLY_FRAME_P = 0; \
+	SCM_TRACE_P = 0; \
+	SCM_RESET_DEBUG_MODE; \
 	if (SCM_CHEAPTRAPS_P)\
 	  {\
 	    tmp = scm_make_debugobj (&debug);\
@@ -1184,7 +1238,7 @@ scm_eval_args (l, env)
 	else\
 	  {\
 	    scm_make_cont (&tmp);\
-	    if (!setjmp (SCM_JMPBUF (tmp)))\
+	    if (!safe_setjmp (SCM_JMPBUF (tmp)))\
 	      scm_ithrow (scm_i_apply_frame, scm_cons2 (tmp, tail, SCM_EOL), 0);\
 	  }\
       }\
@@ -1279,6 +1333,7 @@ scm_deval_args (l, env, lloc)
 #endif
 #endif /* DEVAL */
 
+#define BUILTIN_RPASUBR /* Handle rpsubrs and asubrs without calling apply */
 
 /* SECTION: This is the evaluator.  Like any real monster, it has
  * three heads.  This code is compiled twice.
@@ -1342,21 +1397,6 @@ loopnoap:
   PREP_APPLY (SCM_UNDEFINED, SCM_EOL);
 loop:
 #ifdef DEVAL
-#if 0 /* This will probably never have any practical use ... */
-  if (CHECK_EXIT)
-    {
-      if (SINGLE_STEP || (SCM_TRACE_P && SCM_TRACED_FRAME_P (debug)))
-	{
-	  SINGLE_STEP = 0;
-	  SCM_RESET_DEBUG_MODE;
-	  SCM_CLEAR_TRACED_FRAME (debug);
-	  scm_make_cont (&t.arg1);
-	  if (!setjmp (SCM_JMPBUF (t.arg1)))
-	    scm_ithrow (scm_i_exit_tail, scm_cons (t.arg1, SCM_EOL), 0);
-	}
-    }
-nextframe:
-#endif
   SCM_CLEAR_ARGSREADY (debug);
   if (SCM_OVERFLOWP (debug))
     --debug.info;
@@ -1380,7 +1420,7 @@ start:
 	else
 	  {
 	    scm_make_cont (&t.arg1);
-	    if (setjmp (SCM_JMPBUF (t.arg1)))
+	    if (safe_setjmp (SCM_JMPBUF (t.arg1)))
 	      {
 		x = SCM_THROW_VALUE (t.arg1);
 		if (SCM_IMP (x))
@@ -1706,7 +1746,7 @@ dispatch:
 
 	case (SCM_ISYMNUM (SCM_IM_CONT)):
 	  scm_make_cont (&t.arg1);
-	  if (setjmp (SCM_JMPBUF (t.arg1)))
+	  if (safe_setjmp (SCM_JMPBUF (t.arg1)))
 	    {
 	      SCM val;
 	      val = SCM_THROW_VALUE (t.arg1);
@@ -2086,18 +2126,34 @@ evapply:
 	SCM_ASRTGO (SCM_NULLP (SCM_CDR (x)), wrongnumargs);
 	RETURN (SCM_SUBRF (proc) (t.arg1, arg2, SCM_CAR (SCM_CDR (SCM_CDR (debug.info->a.args)))));
       case scm_tc7_asubr:
-	/*      t.arg1 = SCM_SUBRF(proc)(t.arg1, arg2);
-		while SCM_NIMP(x) {
-		t.arg1 = SCM_SUBRF(proc)(t.arg1, EVALCAR(x, env));
-		x = SCM_CDR(x);
-		}
-		RETURN (t.arg1) */
+#ifdef BUILTIN_RPASUBR
+	t.arg1 = SCM_SUBRF(proc)(t.arg1, arg2);
+	arg2 = SCM_CDR (SCM_CDR (debug.info->a.args));
+	do {
+	  t.arg1 = SCM_SUBRF(proc)(t.arg1, SCM_CAR (arg2));
+	  arg2 = SCM_CDR (arg2);
+	} while (SCM_NIMP (arg2));
+	RETURN (t.arg1)
+#endif /* BUILTIN_RPASUBR */
       case scm_tc7_rpsubr:
+#ifdef BUILTIN_RPASUBR
+	if (SCM_FALSEP (SCM_SUBRF (proc) (t.arg1, arg2)))
+	  RETURN (SCM_BOOL_F)
+	t.arg1 = SCM_CDR (SCM_CDR (debug.info->a.args));
+	do {
+	  if (SCM_FALSEP (SCM_SUBRF (proc) (arg2, SCM_CAR (t.arg1))))
+	    RETURN (SCM_BOOL_F)
+	  arg2 = SCM_CAR (t.arg1);
+	  t.arg1 = SCM_CDR (t.arg1);
+	} while (SCM_NIMP (t.arg1));
+	RETURN (SCM_BOOL_T)
+#else /* BUILTIN_RPASUBR */
 	RETURN (SCM_APPLY (proc, t.arg1, scm_acons (arg2, SCM_CDR (SCM_CDR (debug.info->a.args)), SCM_EOL)))
-	  case scm_tc7_lsubr_2:
-	    RETURN (SCM_SUBRF (proc) (t.arg1, arg2, SCM_CDR (SCM_CDR (debug.info->a.args))))
-	  case scm_tc7_lsubr:
-	    RETURN (SCM_SUBRF (proc) (debug.info->a.args))
+#endif /* BUILTIN_RPASUBR */
+      case scm_tc7_lsubr_2:
+	RETURN (SCM_SUBRF (proc) (t.arg1, arg2, SCM_CDR (SCM_CDR (debug.info->a.args))))
+      case scm_tc7_lsubr:
+	RETURN (SCM_SUBRF (proc) (debug.info->a.args))
 #ifdef CCLO
       case scm_tc7_cclo:
 	goto cclon;
@@ -2114,14 +2170,29 @@ evapply:
 	SCM_ASRTGO (SCM_NULLP (SCM_CDR (x)), wrongnumargs);
 	RETURN (SCM_SUBRF (proc) (t.arg1, arg2, EVALCAR (x, env)));
       case scm_tc7_asubr:
-	/*      t.arg1 = SCM_SUBRF(proc)(t.arg1, arg2);
-		while SCM_NIMP(x) {
-		t.arg1 = SCM_SUBRF(proc)(t.arg1, EVALCAR(x, env));
-		x = SCM_CDR(x);
-		}
-		RETURN (t.arg1) */
+#ifdef BUILTIN_RPASUBR
+	t.arg1 = SCM_SUBRF(proc)(t.arg1, arg2);
+	do {
+	  t.arg1 = SCM_SUBRF(proc)(t.arg1, EVALCAR(x, env));
+	  x = SCM_CDR(x);
+	} while (SCM_NIMP (x));
+	RETURN (t.arg1)
+#endif /* BUILTIN_RPASUBR */
       case scm_tc7_rpsubr:
+#ifdef BUILTIN_RPASUBR
+	if (SCM_FALSEP (SCM_SUBRF (proc) (t.arg1, arg2)))
+	  RETURN (SCM_BOOL_F)
+	do {
+	  t.arg1 = EVALCAR (x, env);
+	  if (SCM_FALSEP (SCM_SUBRF (proc) (arg2, t.arg1)))
+	    RETURN (SCM_BOOL_F)
+	  arg2 = t.arg1;
+	  x = SCM_CDR (x);
+	} while (SCM_NIMP (x));
+	RETURN (SCM_BOOL_T)
+#else /* BUILTIN_RPASUBR */
 	RETURN (SCM_APPLY (proc, t.arg1, scm_acons (arg2, scm_eval_args (x, env), SCM_EOL)));
+#endif /* BUILTIN_RPASUBR */
       case scm_tc7_lsubr_2:
 	RETURN (SCM_SUBRF (proc) (t.arg1, arg2, scm_eval_args (x, env)));
       case scm_tc7_lsubr:
@@ -2158,6 +2229,7 @@ exit:
     if (SCM_EXIT_FRAME_P || (SCM_TRACE_P && SCM_TRACED_FRAME_P (debug)))
       {
 	SCM_EXIT_FRAME_P = 0;
+	SCM_TRACE_P = 0;
 	SCM_RESET_DEBUG_MODE;
 	SCM_CLEAR_TRACED_FRAME (debug);
 	if (SCM_CHEAPTRAPS_P)
@@ -2165,7 +2237,7 @@ exit:
 	else
 	  {
 	    scm_make_cont (&t.arg1);
-	    if (setjmp (SCM_JMPBUF (t.arg1)))
+	    if (safe_setjmp (SCM_JMPBUF (t.arg1)))
 	      {
 		proc = SCM_THROW_VALUE (t.arg1);
 		goto ret;
@@ -2216,20 +2288,24 @@ scm_procedure_documentation (proc)
     }
 }
 
-/* This code processes the 'arg ...' parameters to apply.
+/* This code processes the arguments to apply:
 
    (apply PROC ARG1 ... ARGS)
 
-   The ARG1 ... arguments are consed on to the front of ARGS (which
-   must be a list), and then PROC is applied to the elements of the
+   Given a list (ARG1 ... ARGS), this function conses the ARG1
+   ... arguments onto the front of ARGS, and returns the resulting
+   list.  Note that ARGS is a list; thus, the argument to this
+   function is a list whose last element is a list.
+
+   Apply calls this function, and applies PROC to the elements of the
    result.  apply:nconc2last takes care of building the list of
    arguments, given (ARG1 ... ARGS).
 
-   apply:nconc2last destroys its argument.  On that topic, this code
-   came into my care with the following beautifully cryptic comment on
-   that topic: "This will only screw you if you do (scm_apply
-   scm_apply '( ... ))"  If you know what they're referring to, send
-   me a patch to this comment.  */
+   Rather than do new consing, apply:nconc2last destroys its argument.
+   On that topic, this code came into my care with the following
+   beautifully cryptic comment on that topic: "This will only screw
+   you if you do (scm_apply scm_apply '( ... ))"  If you know what
+   they're referring to, send me a patch to this comment.  */
 
 SCM_PROC(s_nconc2last, "apply:nconc2last", 1, 0, 0, scm_nconc2last);
 
@@ -2275,6 +2351,16 @@ scm_dapply (proc, arg1, args)
 #endif
 
 
+/* Apply a function to a list of arguments.
+
+   This function is exported to the Scheme level as taking two
+   required arguments and a tail argument, as if it were:
+	(lambda (proc arg1 . args) ...)
+   Thus, if you just have a list of arguments to pass to a procedure,
+   pass the list as ARG1, and '() for ARGS.  If you have some fixed
+   args, pass the first as ARG1, then cons any remaining fixed args
+   onto the front of your argument list, and pass that as ARGS.  */
+
 SCM 
 SCM_APPLY (proc, arg1, args)
      SCM proc;
@@ -2298,23 +2384,46 @@ SCM_APPLY (proc, arg1, args)
 #endif
 
   SCM_ASRTGO (SCM_NIMP (proc), badproc);
+
+  /* If ARGS is the empty list, then we're calling apply with only two
+     arguments --- ARG1 is the list of arguments for PROC.  Whatever
+     the case, futz with things so that ARG1 is the first argument to
+     give to PROC (or SCM_UNDEFINED if no args), and ARGS contains the
+     rest.
+
+     Setting the debug apply frame args this way is pretty messy.
+     Perhaps we should store arg1 and args directly in the frame as
+     received, and let scm_frame_arguments unpack them, because that's
+     a relatively rare operation.  This works for now; if the Guile
+     developer archives are still around, see Mikael's post of
+     11-Apr-97.  */
   if (SCM_NULLP (args))
     {
       if (SCM_NULLP (arg1))
-	arg1 = SCM_UNDEFINED;
+	{
+	  arg1 = SCM_UNDEFINED;
+#ifdef DEVAL
+	  debug.vect[0].a.args = SCM_EOL;
+#endif
+	}
       else
 	{
+#ifdef DEVAL
+	  debug.vect[0].a.args = arg1;
+#endif
 	  args = SCM_CDR (arg1);
 	  arg1 = SCM_CAR (arg1);
 	}
     }
   else
     {
-      /*		SCM_ASRTGO(SCM_NIMP(args) && SCM_CONSP(args), wrongnumargs); */
+      /* SCM_ASRTGO(SCM_NIMP(args) && SCM_CONSP(args), wrongnumargs); */
       args = scm_nconc2last (args);
+#ifdef DEVAL
+      debug.vect[0].a.args = scm_cons (arg1, args);
+#endif
     }
 #ifdef DEVAL
-  debug.vect[0].a.args = scm_cons (arg1, args);
   if (SCM_ENTER_FRAME_P)
     {
       SCM tmp;
@@ -2325,7 +2434,7 @@ SCM_APPLY (proc, arg1, args)
       else
 	{
 	  scm_make_cont (&tmp);
-	  if (setjmp (SCM_JMPBUF (tmp)))
+	  if (safe_setjmp (SCM_JMPBUF (tmp)))
 	    goto entap;
 	}
       scm_ithrow (scm_i_enter_frame, scm_cons (tmp, SCM_EOL), 0);
@@ -2472,7 +2581,7 @@ exit:
 	else
 	  {
 	    scm_make_cont (&arg1);
-	    if (setjmp (SCM_JMPBUF (arg1)))
+	    if (safe_setjmp (SCM_JMPBUF (arg1)))
 	      {
 		proc = SCM_THROW_VALUE (arg1);
 		goto ret;
@@ -2906,6 +3015,7 @@ scm_init_eval ()
 
   scm_top_level_lookup_closure_var =
     scm_sysintern("*top-level-lookup-closure*", SCM_BOOL_F);
+  scm_can_use_top_level_lookup_closure_var = 1;
 
   scm_i_and = scm_make_synt ("and", scm_makmmacro, scm_m_and);
   scm_i_begin = scm_make_synt ("begin", scm_makmmacro, scm_m_begin);
