@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -95,12 +95,10 @@ scm_setbuf0 (port)
 {
 #ifndef NOSETBUF
 #ifndef MSDOS
-#ifdef FIONREAD
 #ifndef ultrix
   SCM_SYSCALL (setbuf ((FILE *)SCM_STREAM (port), 0););
 #endif
 #endif
-#endif
 #endif
   return SCM_UNSPECIFIED;
 }
@@ -159,10 +157,13 @@ scm_open_file (filename, modes)
   SCM_SYSCALL (f = fopen (file, mode));
   if (!f)
     {
+      int en = errno;
+
       scm_syserror_msg (s_open_file, "%s: %S",
 			scm_listify (scm_makfrom0str (strerror (errno)),
 				     filename,
-				     SCM_UNDEFINED));
+				     SCM_UNDEFINED),
+			en);
     }
   else
     {
@@ -362,6 +363,18 @@ local_ffwrite (ptr, size, nitems, fp)
   return ffwrite (ptr, size, nitems, fp);
 }
 
+/* On SunOS, there's no declaration for pclose in the headers, so
+   putting it directly in the initializer for scm_pipob doesn't really
+   fly.  We could add an extern declaration for it, but then it'll
+   mismatch on some systems that do have a declaration.  So we just
+   wrap it up this way.  */
+static int
+local_pclose (fp)
+     FILE * fp;
+{
+  return pclose (fp);
+}
+
 
 scm_ptobfuns scm_fptob =
 {
@@ -377,21 +390,20 @@ scm_ptobfuns scm_fptob =
   (int (*) SCM_P ((SCM))) local_fclose
 };
 
-/* {Pipe ports}
- */
+/* {Pipe ports} */
 scm_ptobfuns scm_pipob =
 {
   scm_mark0,
-  0, 				/* replaced by pclose in scm_init_ioext() */
-  0, 				/* replaced by prinpipe in scm_init_ioext() */
+  (int (*) SCM_P ((SCM))) local_pclose,  
+  scm_prinport,
   0,
   (int (*) SCM_P ((int, SCM))) local_fputc,
   (int (*) SCM_P ((char *, SCM))) local_fputs,
   (scm_sizet (*) SCM_P ((char *, scm_sizet, scm_sizet, SCM))) local_ffwrite,
   (int (*) SCM_P ((SCM))) local_fflush,
   (int (*) SCM_P ((SCM))) scm_fgetc,
-  0
-};				/* replaced by pclose in scm_init_ioext() */
+  (int (*) SCM_P ((SCM))) local_pclose
+};
 
 void
 scm_init_fports ()
