@@ -1,5 +1,5 @@
 /* GDB interface for Guile
- * Copyright (C) 1996 Mikael Djurfeldt
+ * Copyright (C) 1996 Free Software Foundation
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -215,7 +215,7 @@ gdb_read (str)
   /* Read one object */
   tok_buf_mark_p = SCM_GC8MARKP (tok_buf);
   SCM_CLRGC8MARK (tok_buf);
-  ans = scm_lreadr (&tok_buf, gdb_input_port, 0, SCM_BOOL_F, &ans);
+  ans = scm_lreadr (&tok_buf, gdb_input_port, &ans);
   if (SCM_GC_P)
     {
       if (SCM_NIMP (ans))
