@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -87,6 +87,7 @@
 #include "ramap.h"
 #include "read.h"
 #include "scmsigs.h"
+#include "script.h"
 #include "sequences.h"
 #include "simpos.h"
 #include "smob.h"
@@ -138,6 +139,8 @@ scm_start_stack (base)
   scm_top_level_lookup_closure_var = SCM_BOOL_F;
   scm_system_transformer = SCM_BOOL_F;
 
+  scm_the_last_stack_var = SCM_BOOL_F;
+  
   /* Create an object to hold the root continuation.
    */
   SCM_NEWCELL (scm_rootcont);
@@ -240,8 +243,19 @@ check_config ()
 static void
 scm_init_standard_ports ()
 {
-  /* I'm not sure why this should be unbuffered when coming from a
-     tty; isn't line buffering more common?  */
+  /* From the SCSH manual:
+
+     It can be useful to turn I/O buffering off in some cases, for
+     example when an I/O stream is to be shared by multiple
+     subprocesses.  For this reason, scsh allocates an unbuffered port
+     for file descriptor 0 at start-up time.
+
+     Because shells frequently share stdin with subprocesses, if the
+     shell does buffered reads, it might ``steal'' input intended for
+     a subprocess.  For this reason, all shells, including sh, csh,
+     and scsh, read stdin unbuffered.  Applications that can tolerate
+     buffered input on stdin can reset \ex{(current-input-port)} to
+     block buffering for higher performance.  */
   scm_def_inp = scm_stdio_to_port (stdin, 
 				   (isatty (fileno (stdin)) ? "r0" : "r"),
 				   "standard input");
@@ -323,7 +337,7 @@ scm_boot_guile (argc, argv, main_func, closure)
   c.argc = argc;
   c.argv = argv;
 
-  return scm_boot_guile_1 (&dummy, &c);
+  scm_boot_guile_1 (&dummy, &c);
 }
 
 
@@ -439,6 +453,7 @@ scm_boot_guile_1 (base, closure)
       scm_init_load_path ();
       scm_init_standard_ports ();
       scm_init_dynamic_linking ();
+      scm_init_script ();
       initialized = 1;
     }
 
