@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995 Free Software Foundation, Inc.
+/*	Copyright (C) 1995, 1996, 1997 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,9 +42,12 @@
 
 
 #include <stdio.h>
-#include "fd.h"
 #include "_scm.h"
+#include "genio.h"
+#include "read.h"
 #include "fports.h"
+#include "unif.h"
+#include "chars.h"
 
 #include "ioext.h"
 
@@ -56,17 +59,107 @@
 #endif
 
 
-SCM_PROC (s_sys_ftell, "ftell", 1, 0, 0, scm_sys_ftell);
+SCM_PROC (s_read_delimited_x, "%read-delimited!", 3, 3, 0, scm_read_delimited_x);
 
 SCM 
-scm_sys_ftell (port)
+scm_read_delimited_x (delims, buf, gobble, port, start, end)
+     SCM delims;
+     SCM buf;
+     SCM gobble;
+     SCM port;
+     SCM start;
+     SCM end;
+{
+  long j;
+  char *cbuf;
+  long cstart;
+  long cend;
+  int c;
+  char *cdelims;
+  int num_delims;
+
+  SCM_ASSERT (SCM_NIMP (delims) && SCM_ROSTRINGP (delims),
+	      delims, SCM_ARG1, s_read_delimited_x);
+  cdelims = SCM_ROCHARS (delims);
+  num_delims = SCM_ROLENGTH (delims);
+  SCM_ASSERT (SCM_NIMP (buf) && SCM_STRINGP (buf),
+	      buf, SCM_ARG2, s_read_delimited_x);
+  cbuf = SCM_CHARS (buf);
+  cend = SCM_LENGTH (buf);
+  if (SCM_UNBNDP (port))
+    port = scm_cur_inp;
+  else
+    {
+      SCM_ASSERT (SCM_NIMP (port) && SCM_OPINPORTP (port),
+		  port, SCM_ARG1, s_read_delimited_x);
+    }
+
+  if (SCM_UNBNDP (start))
+    cstart = 0;
+  else
+    {
+      cstart = scm_num2long (start,
+			     (char *) SCM_ARG5, s_read_delimited_x);
+      if (cstart < 0 || cstart >= cend)
+	scm_out_of_range (s_read_delimited_x, start);
+
+      if (!SCM_UNBNDP (end))
+	{
+	  long tend = scm_num2long (end, (char *) SCM_ARG6,
+				    s_read_delimited_x);
+	  if (tend <= cstart || tend > cend)
+	    scm_out_of_range (s_read_delimited_x, end);
+	  cend = tend;
+	}
+    }
+
+  for (j = cstart; j < cend; j++)
+    {  
+      int k;
+
+      c = scm_gen_getc (port);
+      for (k = 0; k < num_delims; k++)
+	{
+	  if (cdelims[k] == c)
+	    {
+	      if (SCM_FALSEP (gobble))
+		scm_gen_ungetc (c, port);
+
+	      return scm_cons (SCM_MAKICHR (c),
+			       scm_long2num (j - cstart));
+	    }
+	}
+      if (c == EOF)
+	return scm_cons (SCM_EOF_VAL, 
+			 scm_long2num (j - cstart));
+
+      cbuf[j] = c;
+    }
+  return scm_cons (SCM_BOOL_F, scm_long2num (j - cstart));
+}
+
+SCM_PROC (s_write_line, "write-line", 1, 1, 0, scm_write_line);
+
+SCM 
+scm_write_line (obj, port)
+     SCM obj;
+     SCM port;
+{
+  scm_display (obj, port);
+  return scm_newline (port);
+}
+
+SCM_PROC (s_ftell, "ftell", 1, 0, 0, scm_ftell);
+
+SCM 
+scm_ftell (port)
      SCM port;
 {
   long pos;
-  SCM_ASSERT (SCM_NIMP (port) && SCM_OPFPORTP (port), port, SCM_ARG1, s_sys_ftell);
+  SCM_ASSERT (SCM_NIMP (port) && SCM_OPFPORTP (port), port, SCM_ARG1, s_ftell);
   SCM_SYSCALL (pos = ftell ((FILE *)SCM_STREAM (port)));
   if (pos < 0)
-    scm_syserror (s_sys_ftell);
+    scm_syserror (s_ftell);
   if (pos > 0 && SCM_CRDYP (port))
     pos--;
   return scm_long2num (pos);
@@ -74,10 +167,10 @@ scm_sys_ftell (port)
 
 
 
-SCM_PROC (s_sys_fseek, "fseek", 3, 0, 0, scm_sys_fseek);
+SCM_PROC (s_fseek, "fseek", 3, 0, 0, scm_fseek);
 
 SCM 
-scm_sys_fseek (port, offset, whence)
+scm_fseek (port, offset, whence)
      SCM port;
      SCM offset;
      SCM whence;
@@ -85,35 +178,41 @@ scm_sys_fseek (port, offset, whence)
   int rv;
   long loff;
 
-  SCM_ASSERT (SCM_NIMP (port) && SCM_OPFPORTP (port), port, SCM_ARG1, s_sys_fseek);
-  loff = scm_num2long (offset, (char *)SCM_ARG2, s_sys_fseek);
+  SCM_ASSERT (SCM_NIMP (port) && SCM_OPFPORTP (port), port, SCM_ARG1, s_fseek);
+  loff = scm_num2long (offset, (char *)SCM_ARG2, s_fseek);
   SCM_ASSERT (SCM_INUMP (whence) && (SCM_INUM (whence) < 3) && (SCM_INUM (whence) >= 0),
-	  whence, SCM_ARG3, s_sys_fseek);
+	  whence, SCM_ARG3, s_fseek);
   
   SCM_CLRDY (port);			/* Clear ungetted char */
   /* Values of whence are interned in scm_init_ioext.  */
   rv = fseek ((FILE *)SCM_STREAM (port), loff, SCM_INUM (whence));
   if (rv != 0)
-    scm_syserror (s_sys_fseek);
+    scm_syserror (s_fseek);
   return SCM_UNSPECIFIED;
 }
 
 
 
-SCM_PROC (s_sys_freopen, "freopen", 3, 0, 0, scm_sys_freopen);
+SCM_PROC (s_freopen, "freopen", 3, 0, 0, scm_freopen);
 
 SCM 
-scm_sys_freopen (filename, modes, port)
+scm_freopen (filename, modes, port)
      SCM filename;
      SCM modes;
      SCM port;
 {
   FILE *f;
-  SCM_ASSERT (SCM_NIMP (filename) && SCM_STRINGP (filename), filename, SCM_ARG1, s_sys_freopen);
-  SCM_ASSERT (SCM_NIMP (modes) && SCM_STRINGP (modes), modes, SCM_ARG2, s_sys_freopen);
+  SCM_ASSERT (SCM_NIMP (filename) && SCM_ROSTRINGP (filename), filename,
+	      SCM_ARG1, s_freopen);
+  SCM_ASSERT (SCM_NIMP (modes) && SCM_ROSTRINGP (modes), modes, SCM_ARG2,
+	      s_freopen);
+
+  SCM_COERCE_SUBSTR (filename);
+  SCM_COERCE_SUBSTR (modes);
   SCM_DEFER_INTS;
-  SCM_ASSERT (SCM_NIMP (port) && SCM_FPORTP (port), port, SCM_ARG3, s_sys_freopen);
-  SCM_SYSCALL (f = freopen (SCM_CHARS (filename), SCM_CHARS (modes), (FILE *)SCM_STREAM (port)));
+  SCM_ASSERT (SCM_NIMP (port) && SCM_FPORTP (port), port, SCM_ARG3, s_freopen);
+  SCM_SYSCALL (f = freopen (SCM_ROCHARS (filename), SCM_ROCHARS (modes),
+			    (FILE *)SCM_STREAM (port)));
   if (!f)
     {
       SCM p;
@@ -124,9 +223,9 @@ scm_sys_freopen (filename, modes, port)
     }
   else
     {
-      SCM_SETCAR (port, scm_tc16_fport | scm_mode_bits (SCM_CHARS (modes)));
+      SCM_SETCAR (port, scm_tc16_fport | scm_mode_bits (SCM_ROCHARS (modes)));
       SCM_SETSTREAM (port, (SCM)f);
-      SCM_SETCAR (port, scm_tc16_fport | scm_mode_bits (SCM_CHARS (modes)));
+      SCM_SETCAR (port, scm_tc16_fport | scm_mode_bits (SCM_ROCHARS (modes)));
       if (SCM_BUF0 & SCM_CAR (port))
 	scm_setbuf0 (port);
     }
@@ -136,10 +235,10 @@ scm_sys_freopen (filename, modes, port)
 
 
 
-SCM_PROC (s_sys_duplicate_port, "duplicate-port", 2, 0, 0, scm_sys_duplicate_port);
+SCM_PROC (s_duplicate_port, "duplicate-port", 2, 0, 0, scm_duplicate_port);
 
 SCM 
-scm_sys_duplicate_port (oldpt, modes)
+scm_duplicate_port (oldpt, modes)
      SCM oldpt;
      SCM modes;
 {
@@ -147,27 +246,31 @@ scm_sys_duplicate_port (oldpt, modes)
   int newfd;
   FILE *f;
   SCM newpt;
-  SCM_ASSERT (SCM_NIMP (oldpt) && SCM_OPPORTP (oldpt), oldpt, SCM_ARG1, s_sys_duplicate_port);
-  SCM_ASSERT (SCM_NIMP (modes) && SCM_STRINGP (modes), modes, SCM_ARG2, s_sys_duplicate_port);
+  SCM_ASSERT (SCM_NIMP (oldpt) && SCM_OPPORTP (oldpt), oldpt, SCM_ARG1,
+	      s_duplicate_port);
+  SCM_ASSERT (SCM_NIMP (modes) && SCM_ROSTRINGP (modes), modes, SCM_ARG2,
+	      s_duplicate_port);
+
+  SCM_COERCE_SUBSTR (modes);
   SCM_NEWCELL (newpt);
   SCM_DEFER_INTS;
   oldfd = fileno ((FILE *)SCM_STREAM (oldpt));
   if (oldfd == -1)
-    scm_syserror (s_sys_duplicate_port);
+    scm_syserror (s_duplicate_port);
   SCM_SYSCALL (newfd = dup (oldfd));
   if (newfd == -1)
-    scm_syserror (s_sys_duplicate_port);
-  f = fdopen (newfd, SCM_CHARS (modes));
+    scm_syserror (s_duplicate_port);
+  f = fdopen (newfd, SCM_ROCHARS (modes));
   if (!f)
     {
       SCM_SYSCALL (close (newfd));
-      scm_syserror (s_sys_duplicate_port);
+      scm_syserror (s_duplicate_port);
     }
   {
     struct scm_port_table * pt;
     pt = scm_add_to_port_table (newpt);
     SCM_SETPTAB_ENTRY (newpt, pt);
-    SCM_SETCAR (newpt, scm_tc16_fport | scm_mode_bits (SCM_CHARS (modes)));
+    SCM_SETCAR (newpt, scm_tc16_fport | scm_mode_bits (SCM_ROCHARS (modes)));
     if (SCM_BUF0 & SCM_CAR (newpt))
       scm_setbuf0 (newpt);
     SCM_SETSTREAM (newpt, (SCM)f);
@@ -179,65 +282,65 @@ scm_sys_duplicate_port (oldpt, modes)
 
 
 
-SCM_PROC (s_sys_redirect_port, "redirect-port", 2, 0, 0, scm_sys_redirect_port);
+SCM_PROC (s_redirect_port, "redirect-port", 2, 0, 0, scm_redirect_port);
 
 SCM 
-scm_sys_redirect_port (into_pt, from_pt)
+scm_redirect_port (into_pt, from_pt)
      SCM into_pt;
      SCM from_pt;
 {
   int ans, oldfd, newfd;
   SCM_DEFER_INTS;
-  SCM_ASSERT (SCM_NIMP (into_pt) && SCM_OPPORTP (into_pt), into_pt, SCM_ARG1, s_sys_redirect_port);
-  SCM_ASSERT (SCM_NIMP (from_pt) && SCM_OPPORTP (from_pt), from_pt, SCM_ARG2, s_sys_redirect_port);
+  SCM_ASSERT (SCM_NIMP (into_pt) && SCM_OPPORTP (into_pt), into_pt, SCM_ARG1, s_redirect_port);
+  SCM_ASSERT (SCM_NIMP (from_pt) && SCM_OPPORTP (from_pt), from_pt, SCM_ARG2, s_redirect_port);
   oldfd = fileno ((FILE *)SCM_STREAM (into_pt));
   if (oldfd == -1)
-    scm_syserror (s_sys_redirect_port);
+    scm_syserror (s_redirect_port);
   newfd = fileno ((FILE *)SCM_STREAM (from_pt));
   if (newfd == -1)
-    scm_syserror (s_sys_redirect_port);
+    scm_syserror (s_redirect_port);
   SCM_SYSCALL (ans = dup2 (oldfd, newfd));
   if (ans == -1)
-    scm_syserror (s_sys_redirect_port);
+    scm_syserror (s_redirect_port);
   SCM_ALLOW_INTS;
   return SCM_UNSPECIFIED;
 }
 
-SCM_PROC (s_sys_fileno, "fileno", 1, 0, 0, scm_sys_fileno);
+SCM_PROC (s_fileno, "fileno", 1, 0, 0, scm_fileno);
 
 SCM 
-scm_sys_fileno (port)
+scm_fileno (port)
      SCM port;
 {
   int fd;
-  SCM_ASSERT (SCM_NIMP (port) && SCM_OPFPORTP (port), port, SCM_ARG1, s_sys_fileno);
+  SCM_ASSERT (SCM_NIMP (port) && SCM_OPFPORTP (port), port, SCM_ARG1, s_fileno);
   fd = fileno ((FILE *)SCM_STREAM (port));
   if (fd == -1)
-    scm_syserror (s_sys_fileno);
+    scm_syserror (s_fileno);
   return SCM_MAKINUM (fd);
 }
 
-SCM_PROC (s_sys_isatty, "isatty?", 1, 0, 0, scm_sys_isatty_p);
+SCM_PROC (s_isatty, "isatty?", 1, 0, 0, scm_isatty_p);
 
 SCM 
-scm_sys_isatty_p (port)
+scm_isatty_p (port)
      SCM port;
 {
   int rv;
-  SCM_ASSERT (SCM_NIMP (port) && SCM_OPFPORTP (port), port, SCM_ARG1, s_sys_isatty);
+  SCM_ASSERT (SCM_NIMP (port) && SCM_OPFPORTP (port), port, SCM_ARG1, s_isatty);
   rv = fileno ((FILE *)SCM_STREAM (port));
   if (rv == -1)
-    scm_syserror (s_sys_isatty);
+    scm_syserror (s_isatty);
   rv = isatty (rv);
   return  rv ? SCM_BOOL_T : SCM_BOOL_F;
 }
 
 
 
-SCM_PROC (s_sys_fdopen, "fdopen", 2, 0, 0, scm_sys_fdopen);
+SCM_PROC (s_fdopen, "fdopen", 2, 0, 0, scm_fdopen);
 
 SCM
-scm_sys_fdopen (fdes, modes)
+scm_fdopen (fdes, modes)
      SCM fdes;
      SCM modes;
 {
@@ -245,16 +348,18 @@ scm_sys_fdopen (fdes, modes)
   SCM port;
   struct scm_port_table * pt;
 
-  SCM_ASSERT (SCM_INUMP (fdes), fdes, SCM_ARG1, s_sys_fdopen);
-  SCM_ASSERT (SCM_NIMP (modes) && SCM_STRINGP (modes), modes, SCM_ARG2, s_sys_fdopen);
+  SCM_ASSERT (SCM_INUMP (fdes), fdes, SCM_ARG1, s_fdopen);
+  SCM_ASSERT (SCM_NIMP (modes) && SCM_ROSTRINGP (modes), modes, SCM_ARG2,
+	      s_fdopen);
+  SCM_COERCE_SUBSTR (modes);
   SCM_NEWCELL (port);
   SCM_DEFER_INTS;
-  f = fdopen (SCM_INUM (fdes), SCM_CHARS (modes));
+  f = fdopen (SCM_INUM (fdes), SCM_ROCHARS (modes));
   if (f == NULL)
-    scm_syserror (s_sys_fdopen);
+    scm_syserror (s_fdopen);
   pt = scm_add_to_port_table (port);
   SCM_SETPTAB_ENTRY (port, pt);
-  SCM_SETCAR (port, scm_tc16_fport | scm_mode_bits (SCM_CHARS (modes)));
+  SCM_SETCAR (port, scm_tc16_fport | scm_mode_bits (SCM_ROCHARS (modes)));
   if (SCM_BUF0 & SCM_CAR (port))
     scm_setbuf0 (port);
   SCM_SETSTREAM (port, (SCM)f);
@@ -269,10 +374,10 @@ scm_sys_fdopen (fdes, modes)
  *          #t if fdes moved. 
  * MOVE->FDES is implemented in Scheme and calls this primitive.
  */
-SCM_PROC (s_sys_primitive_move_to_fdes, "primitive-move->fdes", 2, 0, 0, scm_sys_primitive_move_to_fdes);
+SCM_PROC (s_primitive_move_to_fdes, "primitive-move->fdes", 2, 0, 0, scm_primitive_move_to_fdes);
 
 SCM
-scm_sys_primitive_move_to_fdes (port, fd)
+scm_primitive_move_to_fdes (port, fd)
      SCM port;
      SCM fd;
 {
@@ -281,8 +386,8 @@ scm_sys_primitive_move_to_fdes (port, fd)
   int new_fd;
   int rv;
 
-  SCM_ASSERT (SCM_NIMP (port) && SCM_OPFPORTP (port), port, SCM_ARG1, s_sys_primitive_move_to_fdes);
-  SCM_ASSERT (SCM_INUMP (fd), fd, SCM_ARG2, s_sys_primitive_move_to_fdes);
+  SCM_ASSERT (SCM_NIMP (port) && SCM_OPFPORTP (port), port, SCM_ARG1, s_primitive_move_to_fdes);
+  SCM_ASSERT (SCM_INUMP (fd), fd, SCM_ARG2, s_primitive_move_to_fdes);
   SCM_DEFER_INTS;
   stream = (FILE *)SCM_STREAM (port);
   old_fd = fileno (stream);
@@ -295,13 +400,16 @@ scm_sys_primitive_move_to_fdes (port, fd)
   scm_evict_ports (new_fd);
   rv = dup2 (old_fd, new_fd);
   if (rv == -1)
-    scm_syserror (s_sys_primitive_move_to_fdes);
+    scm_syserror (s_primitive_move_to_fdes);
   scm_setfileno (stream, new_fd);
   SCM_SYSCALL (close (old_fd));  
   SCM_ALLOW_INTS;
   return SCM_BOOL_T;
 }
 
+#ifdef FD_SETTER
+#define SET_FILE_FD_FIELD(F,D) ((F)->FD_SETTER = (D))
+#endif
 
 void
 scm_setfileno (fs, fd)
@@ -311,15 +419,8 @@ scm_setfileno (fs, fd)
 #ifdef SET_FILE_FD_FIELD
   SET_FILE_FD_FIELD(fs, fd);
 #else
-  Configure could not guess the name of the correct field in a FILE *.
-
-  This function needs to be ported to your system.
-
-  SET_FILE_FD_FIELD should change the descriptor refered to by a stdio
-  stream, and nothing else.
-
-  The way to port this file is to add cases to configure.in.  Search
-  that file for "SET_FILE_FD_FIELD" and follow the examples there.
+  scm_misc_error ("scm_setfileno", "Not fully implemented on this platform",
+		  SCM_EOL);
 #endif
 }
 
@@ -379,80 +480,6 @@ scm_init_ioext ()
   scm_sysintern ("SEEK_CUR", SCM_MAKINUM (SEEK_CUR));
   scm_sysintern ("SEEK_END", SCM_MAKINUM (SEEK_END));
 
-  /* File type/permission bits.  */
-#ifdef S_IRUSR
-  scm_sysintern ("S_IRUSR", SCM_MAKINUM (S_IRUSR));
-#endif
-#ifdef S_IWUSR
-  scm_sysintern ("S_IWUSR", SCM_MAKINUM (S_IWUSR));
-#endif
-#ifdef S_IXUSR
-  scm_sysintern ("S_IXUSR", SCM_MAKINUM (S_IXUSR));
-#endif
-#ifdef S_IRWXU
-  scm_sysintern ("S_IRWXU", SCM_MAKINUM (S_IRWXU));
-#endif
-
-#ifdef S_IRGRP
-  scm_sysintern ("S_IRGRP", SCM_MAKINUM (S_IRGRP));
-#endif
-#ifdef S_IWGRP
-  scm_sysintern ("S_IWGRP", SCM_MAKINUM (S_IWGRP));
-#endif
-#ifdef S_IXGRP
-  scm_sysintern ("S_IXGRP", SCM_MAKINUM (S_IXGRP));
-#endif
-#ifdef S_IRWXG
-  scm_sysintern ("S_IRWXG", SCM_MAKINUM (S_IRWXG));
-#endif
-
-#ifdef S_IROTH
-  scm_sysintern ("S_IROTH", SCM_MAKINUM (S_IROTH));
-#endif
-#ifdef S_IWOTH
-  scm_sysintern ("S_IWOTH", SCM_MAKINUM (S_IWOTH));
-#endif
-#ifdef S_IXOTH
-  scm_sysintern ("S_IXOTH", SCM_MAKINUM (S_IXOTH));
-#endif
-#ifdef S_IRWXO
-  scm_sysintern ("S_IRWXO", SCM_MAKINUM (S_IRWXO));
-#endif
-
-#ifdef S_ISUID
-  scm_sysintern ("S_ISUID", SCM_MAKINUM (S_ISUID));
-#endif
-#ifdef S_ISGID
-  scm_sysintern ("S_ISGID", SCM_MAKINUM (S_ISGID));
-#endif
-#ifdef S_ISVTX
-  scm_sysintern ("S_ISVTX", SCM_MAKINUM (S_ISVTX));
-#endif
-
-#ifdef S_IFMT
-  scm_sysintern ("S_IFMT", SCM_MAKINUM (S_IFMT));
-#endif
-#ifdef S_IFDIR
-  scm_sysintern ("S_IFDIR", SCM_MAKINUM (S_IFDIR));
-#endif
-#ifdef S_IFCHR
-  scm_sysintern ("S_IFCHR", SCM_MAKINUM (S_IFCHR));
-#endif
-#ifdef S_IFBLK
-  scm_sysintern ("S_IFBLK", SCM_MAKINUM (S_IFBLK));
-#endif
-#ifdef S_IFREG
-  scm_sysintern ("S_IFREG", SCM_MAKINUM (S_IFREG));
-#endif
-#ifdef S_IFLNK
-  scm_sysintern ("S_IFLNK", SCM_MAKINUM (S_IFLNK));
-#endif
-#ifdef S_IFSOCK
-  scm_sysintern ("S_IFSOCK", SCM_MAKINUM (S_IFSOCK));
-#endif
-#ifdef S_IFIFO
-  scm_sysintern ("S_IFIFO", SCM_MAKINUM (S_IFIFO));
-#endif
 #include "ioext.x"
 }
 
