@@ -99,6 +99,7 @@
 #include "libguile/read.h"
 #include "libguile/root.h"
 #include "libguile/scmsigs.h"
+#include "libguile/script.h"
 #include "libguile/sequences.h"
 #include "libguile/simpos.h"
 #include "libguile/snarf.h"
@@ -122,7 +123,7 @@
 #include "libguile/vports.h"
 #include "libguile/weaks.h"
 #ifdef USE_THREADS
-#include "libguile/../threads/threads.h"
+#include "libguile/threads.h"
 #endif
 
 
