@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996 Mikael Djurfeldt
+/*	Copyright (C) 1995,1996 Free Software Foundation
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -211,8 +211,7 @@ scm_init_opts (func, options, n)
 
   for (i = 0; i < n; ++i)
     {
-      options[i].name =	(char *) SCM_CAR (scm_sysintern (options[i].name,
-							 SCM_UNDEFINED));
+      options[i].name =	(char *) SCM_CAR (scm_sysintern0 (options[i].name));
       options[i].doc = (char *) scm_permanent_object (scm_take0str
 						      (options[i].doc));
     }
