@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -135,7 +135,7 @@ scm_fflush (port)
 
 
 
-SCM_PROC(s_char_ready_p, "char-ready?", 1, 0, 0, scm_char_ready_p);
+SCM_PROC(s_char_ready_p, "char-ready?", 0, 1, 0, scm_char_ready_p);
 
 SCM 
 scm_char_ready_p (port)
@@ -153,24 +153,6 @@ scm_char_ready_p (port)
 }
 
 
-
-
-SCM_PROC (s_ungetc_char_ready_p, "ungetc-char-ready?", 1, 0, 0, scm_ungetc_char_ready_p);
-
-SCM 
-scm_ungetc_char_ready_p (port)
-     SCM port;
-{
-  if (SCM_UNBNDP (port))
-    port = scm_cur_inp;
-  else
-    SCM_ASSERT (SCM_NIMP (port) && SCM_OPINPORTP (port), port, SCM_ARG1, s_ungetc_char_ready_p);
-  return (SCM_CRDYP (port)
-	  ? SCM_BOOL_T
-	  : SCM_BOOL_F);
-}
-
-
 
 
 
@@ -299,11 +281,11 @@ scm_remove_from_port_table (port)
   scm_port_table_size--;
 }
 
-#ifdef DEBUG
+#ifdef GUILE_DEBUG
 /* Undocumented functions for debugging.  */
 /* Return the number of ports in the table.  */
-static char s_pt_size[] = "pt-size";
 
+SCM_PROC(s_pt_size, "pt-size", 0, 0, 0, scm_pt_size);
 SCM
 scm_pt_size ()
 {
@@ -311,8 +293,7 @@ scm_pt_size ()
 }
 
 /* Return the ith member of the port table.  */
-static char s_pt_member[] = "pt-member";
-
+SCM_PROC(s_pt_member, "pt-member", 1, 0, 0, scm_pt_member);
 SCM
 scm_pt_member (member)
      SCM member;
@@ -834,9 +815,10 @@ SCM
 scm_sys_make_void_port (mode)
      SCM mode;
 {
-  SCM_ASSERT (SCM_NIMP (mode) && SCM_STRINGP (mode), mode,
+  SCM_ASSERT (SCM_NIMP (mode) && SCM_ROSTRINGP (mode), mode,
 	      SCM_ARG1, s_sys_make_void_port);
 
+  SCM_COERCE_SUBSTR (mode);
   return scm_void_port (SCM_ROCHARS (mode));
 }
 
