@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995, 1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995, 1996, 1997 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,12 +43,8 @@
 #include <stdio.h>
 #include "_scm.h"
 #include "fports.h"
-#include "genio.h"
 #include "scmsigs.h"
-#include "read.h"
-#include "unif.h"
 #include "feature.h"
-#include "sequences.h"
 
 #include "posix.h"
 
@@ -79,10 +75,6 @@ extern char *ttyname();
 #include <libc.h>
 #endif
 
-#ifdef HAVE_SYS_SELECT_H
-#include <sys/select.h>
-#endif
-
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -101,23 +93,6 @@ extern char *ttyname();
 
 #include <signal.h>
 
-#ifdef FD_SET
-
-#define SELECT_TYPE fd_set
-#define SELECT_SET_SIZE FD_SETSIZE
-
-#else /* no FD_SET */
-
-/* Define the macros to access a single-int bitmap of descriptors.  */
-#define SELECT_SET_SIZE 32
-#define SELECT_TYPE int
-#define FD_SET(n, p) (*(p) |= (1 << (n)))
-#define FD_CLR(n, p) (*(p) &= ~(1 << (n)))
-#define FD_ISSET(n, p) (*(p) & (1 << (n)))
-#define FD_ZERO(p) (*(p) = 0)
-
-#endif /* no FD_SET */
-
 extern FILE *popen ();
 extern char ** environ;
 
@@ -141,8 +116,6 @@ extern char ** environ;
 # endif
 #endif
 
-char *strptime ();
-
 #ifdef HAVE_SETLOCALE
 #include <locale.h>
 #endif
@@ -286,6 +259,11 @@ scm_getpwuid (user)
     {
       SCM_DEFER_INTS;
       SCM_SYSCALL (entry = getpwent ());
+      if (! entry)
+	{
+	  SCM_ALLOW_INTS;
+	  return SCM_BOOL_F;
+	}
     }
   else if (SCM_INUMP (user))
     {
@@ -351,15 +329,22 @@ scm_getgrgid (name)
   ve = SCM_VELTS (result);
   SCM_DEFER_INTS;
   if (SCM_UNBNDP (name) || (name == SCM_BOOL_F))
-    SCM_SYSCALL (entry = getgrent ());
+    {
+      SCM_SYSCALL (entry = getgrent ());
+      if (! entry)
+	{
+	  SCM_ALLOW_INTS;
+	  return SCM_BOOL_F;
+	}
+    }
   else if (SCM_INUMP (name))
     SCM_SYSCALL (entry = getgrgid (SCM_INUM (name)));
   else
     {
-      SCM_ASSERT (SCM_NIMP (name) && SCM_STRINGP (name), name, SCM_ARG1, s_getgrgid);
-      if (SCM_SUBSTRP (name))
-	name = scm_makfromstr (SCM_ROCHARS (name), SCM_ROLENGTH (name), 0);
-      SCM_SYSCALL (entry = getgrnam (SCM_CHARS (name)));
+      SCM_ASSERT (SCM_NIMP (name) && SCM_ROSTRINGP (name), name, SCM_ARG1,
+		  s_getgrgid);
+      SCM_COERCE_SUBSTR (name);
+      SCM_SYSCALL (entry = getgrnam (SCM_ROCHARS (name)));
     }
   if (!entry)
     scm_syserror (s_getgrgid);
@@ -437,7 +422,55 @@ scm_waitpid (pid, options)
 #endif
 }
 
+SCM_PROC (s_status_exit_val, "status:exit-val", 1, 0, 0, scm_status_exit_val);
+SCM
+scm_status_exit_val (status)
+     SCM status;
+{
+  int lstatus;
 
+  SCM_ASSERT (SCM_INUMP (status), status, SCM_ARG1,s_status_exit_val);
+
+  /* On Ultrix, the WIF... macros assume their argument is an lvalue;
+     go figure.  SCM_INUM does not yield an lvalue.  */
+  lstatus = SCM_INUM (status);
+  if (WIFEXITED (lstatus))
+    return (SCM_MAKINUM (WEXITSTATUS (lstatus)));
+  else
+    return SCM_BOOL_F;
+}
+
+SCM_PROC (s_status_term_sig, "status:term-sig", 1, 0, 0, scm_status_term_sig);
+SCM
+scm_status_term_sig (status)
+     SCM status;
+{
+  int lstatus;
+
+  SCM_ASSERT (SCM_INUMP (status), status, SCM_ARG1,s_status_term_sig);
+
+  lstatus = SCM_INUM (status);
+  if (WIFSIGNALED (lstatus))
+    return SCM_MAKINUM (WTERMSIG (lstatus));
+  else
+    return SCM_BOOL_F;
+}
+
+SCM_PROC (s_status_stop_sig, "status:stop-sig", 1, 0, 0, scm_status_stop_sig);
+SCM
+scm_status_stop_sig (status)
+     SCM status;
+{
+  int lstatus;
+
+  SCM_ASSERT (SCM_INUMP (status), status, SCM_ARG1,s_status_stop_sig);
+
+  lstatus = SCM_INUM (status);
+  if (WIFSTOPPED (lstatus))
+    return SCM_MAKINUM (WSTOPSIG (lstatus));
+  else
+    return SCM_BOOL_F;
+}
 
 SCM_PROC (s_getppid, "getppid", 0, 0, 0, scm_getppid);
 
@@ -701,8 +734,8 @@ scm_convert_exec_args (args)
       scm_sizet len;
       char *dst;
       char *src;
-      SCM_ASSERT (SCM_NIMP (SCM_CAR (args)) && SCM_ROSTRINGP (SCM_CAR (args)), SCM_CAR (args),
-	      "wrong type in SCM_ARG", "exec arg");
+      SCM_ASSERT (SCM_NIMP (SCM_CAR (args)) && SCM_ROSTRINGP (SCM_CAR (args)),
+		  SCM_CAR (args), "wrong type in SCM_ARG", "exec arg");
       len = 1 + SCM_ROLENGTH (SCM_CAR (args));
       dst = (char *) scm_must_malloc ((long) len, s_ttyname);
       src = SCM_ROCHARS (SCM_CAR (args));
@@ -742,9 +775,11 @@ scm_execlp (args)
 {
   char **execargv;
   SCM filename = SCM_CAR (args);
-  SCM_ASSERT (SCM_NIMP (filename) && SCM_ROSTRINGP (filename), filename, SCM_ARG1, s_execlp);
+  SCM_ASSERT (SCM_NIMP (filename) && SCM_ROSTRINGP (filename), filename,
+	      SCM_ARG1, s_execlp);
   if (SCM_SUBSTRP (filename))
-    filename = scm_makfromstr (SCM_ROCHARS (filename), SCM_ROLENGTH (filename), 0);
+    filename = scm_makfromstr (SCM_ROCHARS (filename),
+			       SCM_ROLENGTH (filename), 0);
   args = SCM_CDR (args);
   execargv = scm_convert_exec_args (args);
   execvp (SCM_ROCHARS (filename), execargv);
@@ -753,8 +788,7 @@ scm_execlp (args)
   return SCM_BOOL_F;
 }
 
-/* Flushing streams etc., is not done here.  */
-SCM_PROC (s_fork, "fork", 0, 0, 0, scm_fork);
+SCM_PROC (s_fork, "primitive-fork", 0, 0, 0, scm_fork);
 
 SCM
 scm_fork()
@@ -776,8 +810,9 @@ scm_uname ()
   struct utsname buf;
   SCM ans = scm_make_vector(SCM_MAKINUM(5), SCM_UNSPECIFIED, SCM_BOOL_F);
   SCM *ve = SCM_VELTS (ans);
+  SCM_DEFER_INTS;
   if (uname (&buf))
-    return SCM_MAKINUM (errno);
+    scm_syserror (s_uname);
   ve[0] = scm_makfrom0str (buf.sysname);
   ve[1] = scm_makfrom0str (buf.nodename);
   ve[2] = scm_makfrom0str (buf.release);
@@ -787,6 +822,7 @@ scm_uname ()
    a linux special?
   ve[5] = scm_makfrom0str (buf.domainname);
 */
+  SCM_ALLOW_INTS;
   return ans;
 #else
   scm_sysmissing (s_uname);
@@ -811,17 +847,20 @@ scm_environ (env)
       SCM_ASSERT (SCM_NULLP (env) || (SCM_NIMP (env) && SCM_CONSP (env)),
 	      env, SCM_ARG1, s_environ);
       num_strings = scm_ilength (env);
-      new_environ = (char **) scm_must_malloc ((num_strings + 1)
-					       * sizeof (char *),
-					       s_environ);
+      new_environ = (char **) malloc ((num_strings + 1) * sizeof (char *));
+      if (new_environ == NULL)
+	scm_memory_error (s_environ);
       while (SCM_NNULLP (env))
 	{
 	  int len;
 	  char *src;
-	  SCM_ASSERT (SCM_NIMP (SCM_CAR (env)) && SCM_ROSTRINGP (SCM_CAR (env)), env, SCM_ARG1,
-		  s_environ);
+	  SCM_ASSERT (SCM_NIMP (SCM_CAR (env))
+		      && SCM_ROSTRINGP (SCM_CAR (env)),
+		      env, SCM_ARG1, s_environ);
 	  len = 1 + SCM_ROLENGTH (SCM_CAR (env));
-	  new_environ[i] = scm_must_malloc ((long) len, s_environ);
+	  new_environ[i] = malloc ((long) len);
+	  if (new_environ[i] == NULL)
+	    scm_memory_error (s_environ);
 	  src = SCM_ROCHARS (SCM_CAR (env));
 	  while (len--) 
 	    new_environ[i][len] = src[len];
@@ -838,8 +877,8 @@ scm_environ (env)
 	if (!first)
 	  {
 	    for (ep = environ; *ep != NULL; ep++)
-	      scm_must_free (*ep);
-	    scm_must_free ((char *) environ);
+	      free (*ep);
+	    free ((char *) environ);
 	  }
 	first = 0;
       }
@@ -871,10 +910,13 @@ scm_open_pipe (pipestr, modes)
   register SCM z;
   struct scm_port_table * pt;
 
-  SCM_ASSERT (SCM_NIMP (pipestr) && SCM_ROSTRINGP (pipestr), pipestr, SCM_ARG1, s_open_pipe);
+  SCM_ASSERT (SCM_NIMP (pipestr) && SCM_ROSTRINGP (pipestr), pipestr,
+	      SCM_ARG1, s_open_pipe);
   if (SCM_SUBSTRP (pipestr))
-    pipestr = scm_makfromstr (SCM_ROCHARS (pipestr), SCM_ROLENGTH (pipestr), 0);
-  SCM_ASSERT (SCM_NIMP (modes) && SCM_ROSTRINGP (modes), modes, SCM_ARG2, s_open_pipe);
+    pipestr = scm_makfromstr (SCM_ROCHARS (pipestr),
+			      SCM_ROLENGTH (pipestr), 0);
+  SCM_ASSERT (SCM_NIMP (modes) && SCM_ROSTRINGP (modes), modes, SCM_ARG2,
+	      s_open_pipe);
   if (SCM_SUBSTRP (modes))
     modes = scm_makfromstr (SCM_ROCHARS (modes), SCM_ROLENGTH (modes), 0);
   SCM_NEWCELL (z);
@@ -893,26 +935,24 @@ scm_open_pipe (pipestr, modes)
   return z;
 }
 
+SCM_PROC (s_close_pipe, "close-pipe", 1, 0, 0, scm_close_pipe);
 
-SCM_PROC (s_open_input_pipe, "open-input-pipe", 1, 0, 0, scm_open_input_pipe);
-
-SCM
-scm_open_input_pipe(pipestr)
-     SCM pipestr;
+SCM 
+scm_close_pipe (port)
+     SCM port;
 {
-  return scm_open_pipe(pipestr, scm_makfromstr("r", (sizeof "r")-1, 0));
+  int rv;
+
+  SCM_ASSERT (SCM_NIMP (port) && SCM_TYP16(port) == scm_tc16_pipe 
+	      && SCM_OPENP (port), port, SCM_ARG1, s_close_pipe);
+  SCM_DEFER_INTS;
+  rv = pclose ((FILE *) SCM_STREAM (port));
+  if (rv == -1)
+    scm_syserror (s_close_pipe);
+  SCM_ALLOW_INTS;
+  return SCM_MAKINUM (rv);
 }
 
-SCM_PROC (s_open_output_pipe, "open-output-pipe", 1, 0, 0, scm_open_output_pipe);
-
-SCM
-scm_open_output_pipe(pipestr)
-     SCM pipestr;
-{
-  return scm_open_pipe(pipestr, scm_makfromstr("w", (sizeof "w")-1, 0));
-}
-
-
 SCM_PROC (s_utime, "utime", 1, 2, 0, scm_utime);
 
 SCM 
@@ -924,8 +964,10 @@ scm_utime (pathname, actime, modtime)
   int rv;
   struct utimbuf utm_tmp;
 
-  SCM_ASSERT (SCM_NIMP (pathname) && SCM_STRINGP (pathname), pathname, SCM_ARG1, s_utime);
+  SCM_ASSERT (SCM_NIMP (pathname) && SCM_ROSTRINGP (pathname), pathname,
+	      SCM_ARG1, s_utime);
 
+  SCM_COERCE_SUBSTR (pathname);
   if (SCM_UNBNDP (actime))
     SCM_SYSCALL (time (&utm_tmp.actime));
   else
@@ -936,7 +978,7 @@ scm_utime (pathname, actime, modtime)
   else
     utm_tmp.modtime = scm_num2ulong (modtime, (char *) SCM_ARG3, s_utime);
 
-  SCM_SYSCALL (rv = utime (SCM_CHARS (pathname), &utm_tmp));
+  SCM_SYSCALL (rv = utime (SCM_ROCHARS (pathname), &utm_tmp));
   if (rv != 0)
     scm_syserror (s_utime);
   return SCM_UNSPECIFIED;
@@ -951,7 +993,8 @@ scm_access (path, how)
 {
   int rv;
 
-  SCM_ASSERT (SCM_NIMP (path) && SCM_ROSTRINGP (path), path, SCM_ARG1, s_access);
+  SCM_ASSERT (SCM_NIMP (path) && SCM_ROSTRINGP (path), path, SCM_ARG1,
+	      s_access);
   if (SCM_SUBSTRP (path))
     path = scm_makfromstr (SCM_ROCHARS (path), SCM_ROLENGTH (path), 0);
   SCM_ASSERT (SCM_INUMP (how), how, SCM_ARG2, s_access);
@@ -973,128 +1016,20 @@ SCM
 scm_putenv (str)
      SCM str;
 {
-#ifdef HAVE_PUTENV
   int rv;
+  char *ptr;
 
-  SCM_ASSERT (SCM_NIMP (str) && SCM_STRINGP (str), str, SCM_ARG1, s_putenv);
-  rv = putenv (SCM_CHARS (str));
+  SCM_ASSERT (SCM_NIMP (str) && SCM_ROSTRINGP (str), str, SCM_ARG1, s_putenv);
+  /* must make a new copy to be left in the environment, safe from gc.  */
+  ptr = malloc (SCM_LENGTH (str) + 1);
+  if (ptr == NULL)
+    scm_memory_error (s_putenv);
+  strncpy (ptr, SCM_ROCHARS (str), SCM_LENGTH (str));
+  ptr[SCM_LENGTH(str)] = 0;
+  rv = putenv (ptr);
   if (rv < 0)
     scm_syserror (s_putenv);
   return SCM_UNSPECIFIED;
-#else
-  scm_sysmissing (s_putenv);
-  /* not reached.  */
-  return SCM_BOOL_F;
-#endif
-}
-
-SCM_PROC (s_read_line, "read-line", 0, 2, 0, scm_read_line);
-
-SCM 
-scm_read_line (port, include_terminator)
-     SCM port;
-     SCM include_terminator;
-{
-  register int c;
-  register int j = 0;
-  scm_sizet len = 30;
-  SCM tok_buf;
-  register char *p;
-  int include;
-
-  tok_buf = scm_makstr ((long) len, 0);
-  p = SCM_CHARS (tok_buf);
-  if (SCM_UNBNDP (port))
-    port = scm_cur_inp;
-  else
-    SCM_ASSERT (SCM_NIMP (port) && SCM_OPINPORTP (port), port, SCM_ARG1, s_read_line);
-
-  if (SCM_UNBNDP (include_terminator))
-    include = 0;
-  else
-    include = SCM_NFALSEP (include_terminator);
-
-  if (EOF == (c = scm_gen_getc (port)))
-    return SCM_EOF_VAL;
-  while (1)
-    {
-      switch (c)
-	{
-	case SCM_LINE_INCREMENTORS:
-	  if (j >= len)
-	    {
-	      p = scm_grow_tok_buf (&tok_buf);
-	      len = SCM_LENGTH (tok_buf);
-	    }
-	  p[j++] = c;
-	  /* fallthrough */
-	case EOF:
-	  if (len == j)
-	    return tok_buf;
-	  return scm_vector_set_length_x (tok_buf, (SCM) SCM_MAKINUM (j));
-
-	default:
-	  if (j >= len)
-	    {
-	      p = scm_grow_tok_buf (&tok_buf);
-	      len = SCM_LENGTH (tok_buf);
-	    }
-	  p[j++] = c;
-	  c = scm_gen_getc (port);
-	  break;
-	}
-    }
-}
-
-SCM_PROC (s_read_line_x, "read-line!", 1, 1, 0, scm_read_line_x);
-
-SCM 
-scm_read_line_x (str, port)
-     SCM str;
-     SCM port;
-{
-  register int c;
-  register int j = 0;
-  register char *p;
-  scm_sizet len;
-  SCM_ASSERT (SCM_NIMP (str) && SCM_STRINGP (str), str, SCM_ARG1, s_read_line_x);
-  p = SCM_CHARS (str);
-  len = SCM_LENGTH (str);
-  if SCM_UNBNDP
-    (port) port = scm_cur_inp;
-  else
-    SCM_ASSERT (SCM_NIMP (port) && SCM_OPINPORTP (port), port, SCM_ARG2, s_read_line_x);
-  c = scm_gen_getc (port);
-  if (EOF == c)
-    return SCM_EOF_VAL;
-  while (1)
-    {
-      switch (c)
-	{
-	case SCM_LINE_INCREMENTORS:
-	case EOF:
-	  return SCM_MAKINUM (j);
-	default:
-	  if (j >= len)
-	    {
-	      scm_gen_ungetc (c, port);
-	      return SCM_BOOL_F;
-	    }
-	  p[j++] = c;
-	  c = scm_gen_getc (port);
-	}
-    }
-}
-
-SCM_PROC (s_write_line, "write-line", 1, 1, 0, scm_write_line);
-
-SCM 
-scm_write_line (obj, port)
-     SCM obj;
-     SCM port;
-{
-  scm_display (obj, port);
-  return scm_newline (port);
 }
 
 SCM_PROC (s_setlocale, "setlocale", 1, 1, 0, scm_setlocale);
@@ -1115,8 +1050,10 @@ scm_setlocale (category, locale)
     }
   else
     {
-      SCM_ASSERT (SCM_NIMP (locale) && SCM_STRINGP (locale), locale, SCM_ARG2, s_setlocale);
-      clocale = SCM_CHARS (locale);
+      SCM_ASSERT (SCM_NIMP (locale) && SCM_ROSTRINGP (locale), locale,
+		  SCM_ARG2, s_setlocale);
+      SCM_COERCE_SUBSTR (locale);
+      clocale = SCM_ROCHARS (locale);
     }
 
   rv = setlocale (SCM_INUM (category), clocale);
@@ -1130,138 +1067,50 @@ scm_setlocale (category, locale)
 #endif
 }
 
-SCM_PROC (s_strftime, "strftime", 2, 0, 0, scm_strftime);
+SCM_PROC (s_mknod, "mknod", 4, 0, 0, scm_mknod);
 
 SCM
-scm_strftime (format, stime)
-     SCM format;
-     SCM stime;
-{
-  struct tm t;
-
-  char *tbuf;
-  int n;
-  int size = 50;
-  char *fmt;
-  int len;
-
-  SCM_ASSERT (SCM_NIMP (format) && SCM_STRINGP (format), format, SCM_ARG1, s_strftime);
-  SCM_ASSERT (SCM_NIMP (stime) && SCM_VECTORP (stime) && scm_obj_length (stime) == 9,
-	  stime, SCM_ARG2, s_strftime);
-
-  fmt = SCM_ROCHARS (format);
-  len = SCM_ROLENGTH (format);
-
-#define tm_deref scm_num2long (SCM_VELTS (stime)[n++], (char *)SCM_ARG2, s_strftime)
-  n = 0;
-  t.tm_sec = tm_deref;
-  t.tm_min = tm_deref;
-  t.tm_hour = tm_deref;
-  t.tm_mday = tm_deref;
-  t.tm_mon = tm_deref;
-  t.tm_year = tm_deref;
-  /* not used by mktime.
-     t.tm_wday = tm_deref;
-     t.tm_yday = tm_deref; */
-  t.tm_isdst = tm_deref;
-#undef tm_deref
-
-  /* fill in missing fields and set the timezone.  */
-  mktime (&t);
-
-  tbuf = scm_must_malloc (size, s_strftime);
-  while ((len = strftime (tbuf, size, fmt, &t)) == size)
-    {
-      scm_must_free (tbuf);
-      size *= 2;
-      tbuf = scm_must_malloc (size, s_strftime);
-    }
-  return scm_makfromstr (tbuf, len, 0);
-}
-
-SCM_PROC (s_strptime, "strptime", 2, 0, 0, scm_strptime);
-
-SCM
-scm_strptime (format, string)
-     SCM format;
-     SCM string;
-{
-#ifdef HAVE_STRPTIME
-  SCM stime;
-  struct tm t;
-
-  char *fmt, *str, *rest;
-  int n;
-
-  SCM_ASSERT (SCM_NIMP (format) && SCM_ROSTRINGP (format), format, SCM_ARG1, s_strptime);
-  if (SCM_SUBSTRP (format))
-    format =  scm_makfromstr (SCM_ROCHARS (format), SCM_ROLENGTH (format), 0);
-  SCM_ASSERT (SCM_NIMP (string) && SCM_ROSTRINGP (string), string, SCM_ARG2, s_strptime);
-  if (SCM_SUBSTRP (string))
-    string =  scm_makfromstr (SCM_ROCHARS (string), SCM_ROLENGTH (string), 0);
-
-  fmt = SCM_CHARS (format);
-  str = SCM_CHARS (string);
-
-  /* initialize the struct tm */
-#define tm_init(field) t.field = 0
-  tm_init (tm_sec);
-  tm_init (tm_min);
-  tm_init (tm_hour);
-  tm_init (tm_mday);
-  tm_init (tm_mon);
-  tm_init (tm_year);
-  tm_init (tm_wday);
-  tm_init (tm_yday);
-  tm_init (tm_isdst);
-#undef tm_init
-
-  SCM_DEFER_INTS;
-  rest = strptime (str, fmt, &t);
-  SCM_ALLOW_INTS;
-
-  if (rest == NULL)
-    scm_syserror (s_strptime);
-
-  stime = scm_make_vector (SCM_MAKINUM (9), scm_long2num (0), SCM_UNDEFINED);
-
-#define stime_set(val) scm_vector_set_x (stime, SCM_MAKINUM (n++), scm_long2num (t.val));
-  n = 0;
-  stime_set (tm_sec);
-  stime_set (tm_min);
-  stime_set (tm_hour);
-  stime_set (tm_mday);
-  stime_set (tm_mon);
-  stime_set (tm_year);
-  stime_set (tm_wday);
-  stime_set (tm_yday);
-  stime_set (tm_isdst);
-#undef stime_set
-
-  return scm_cons (stime, scm_makfrom0str (rest));
-#else
-  scm_sysmissing (s_strptime);
-  /* not reached.  */
-  return SCM_BOOL_F;
-#endif
-}
-
-SCM_PROC (s_mknod, "mknod", 3, 0, 0, scm_mknod);
-
-SCM
-scm_mknod(path, mode, dev)
+scm_mknod(path, type, perms, dev)
      SCM path;
-     SCM mode;
+     SCM type;
+     SCM perms;
      SCM dev;
 {
 #ifdef HAVE_MKNOD
   int val;
-  SCM_ASSERT(SCM_NIMP(path) && SCM_STRINGP(path), path, SCM_ARG1, s_mknod);
-  SCM_ASSERT(SCM_INUMP(mode), mode, SCM_ARG2, s_mknod);
-  SCM_ASSERT(SCM_INUMP(dev), dev, SCM_ARG3, s_mknod);
-  SCM_SYSCALL(val = mknod(SCM_CHARS(path), SCM_INUM(mode), SCM_INUM(dev)));
+  char *p;
+  int ctype;
+
+  SCM_ASSERT (SCM_NIMP(path) && SCM_ROSTRINGP(path), path, SCM_ARG1, s_mknod);
+  SCM_ASSERT (SCM_NIMP(type) && SCM_SYMBOLP (type), type, SCM_ARG2, s_mknod);
+  SCM_ASSERT (SCM_INUMP (perms), perms, SCM_ARG3, s_mknod);
+  SCM_ASSERT (SCM_INUMP(dev), dev, SCM_ARG4, s_mknod);
+  SCM_COERCE_SUBSTR (path);
+
+  p = SCM_CHARS (type);
+  if (strcmp (p, "regular") == 0)
+    ctype = S_IFREG;
+  else if (strcmp (p, "directory") == 0)
+    ctype = S_IFDIR;
+  else if (strcmp (p, "symlink") == 0)
+    ctype = S_IFLNK;
+  else if (strcmp (p, "block-special") == 0)
+    ctype = S_IFBLK;
+  else if (strcmp (p, "char-special") == 0)
+    ctype = S_IFCHR;
+  else if (strcmp (p, "fifo") == 0)
+    ctype = S_IFIFO;
+  else if (strcmp (p, "socket") == 0)
+    ctype = S_IFSOCK;
+  else
+    scm_out_of_range (s_mknod, type);
+
+  SCM_DEFER_INTS;
+  SCM_SYSCALL (val = mknod(SCM_ROCHARS(path), ctype | SCM_INUM (perms),
+			   SCM_INUM (dev)));
   if (val != 0)
     scm_syserror (s_mknod);
+  SCM_ALLOW_INTS;
   return SCM_UNSPECIFIED;
 #else
   scm_sysmissing (s_mknod);
@@ -1327,109 +1176,6 @@ scm_init_posix ()
   scm_sysintern ("WUNTRACED", SCM_MAKINUM (WUNTRACED));
 #endif
 
-#ifdef EINTR
-  scm_sysintern ("EINTR", SCM_MAKINUM (EINTR));
-#endif
-
-#ifdef SIGHUP
-  scm_sysintern ("SIGHUP", SCM_MAKINUM (SIGHUP));
-#endif
-#ifdef SIGINT
-  scm_sysintern ("SIGINT", SCM_MAKINUM (SIGINT));
-#endif
-#ifdef SIGQUIT
-  scm_sysintern ("SIGQUIT", SCM_MAKINUM (SIGQUIT));
-#endif
-#ifdef SIGILL
-  scm_sysintern ("SIGILL", SCM_MAKINUM (SIGILL));
-#endif
-#ifdef SIGTRAP
-  scm_sysintern ("SIGTRAP", SCM_MAKINUM (SIGTRAP));
-#endif
-#ifdef SIGABRT
-  scm_sysintern ("SIGABRT", SCM_MAKINUM (SIGABRT));
-#endif
-#ifdef SIGIOT
-  scm_sysintern ("SIGIOT", SCM_MAKINUM (SIGIOT));
-#endif
-#ifdef SIGBUS
-  scm_sysintern ("SIGBUS", SCM_MAKINUM (SIGBUS));
-#endif
-#ifdef SIGFPE
-  scm_sysintern ("SIGFPE", SCM_MAKINUM (SIGFPE));
-#endif
-#ifdef SIGKILL
-  scm_sysintern ("SIGKILL", SCM_MAKINUM (SIGKILL));
-#endif
-#ifdef SIGUSR1
-  scm_sysintern ("SIGUSR1", SCM_MAKINUM (SIGUSR1));
-#endif
-#ifdef SIGSEGV
-  scm_sysintern ("SIGSEGV", SCM_MAKINUM (SIGSEGV));
-#endif
-#ifdef SIGUSR2
-  scm_sysintern ("SIGUSR2", SCM_MAKINUM (SIGUSR2));
-#endif
-#ifdef SIGPIPE
-  scm_sysintern ("SIGPIPE", SCM_MAKINUM (SIGPIPE));
-#endif
-#ifdef SIGALRM
-  scm_sysintern ("SIGALRM", SCM_MAKINUM (SIGALRM));
-#endif
-#ifdef SIGTERM
-  scm_sysintern ("SIGTERM", SCM_MAKINUM (SIGTERM));
-#endif
-#ifdef SIGSTKFLT
-  scm_sysintern ("SIGSTKFLT", SCM_MAKINUM (SIGSTKFLT));
-#endif
-#ifdef SIGCHLD
-  scm_sysintern ("SIGCHLD", SCM_MAKINUM (SIGCHLD));
-#endif
-#ifdef SIGCONT
-  scm_sysintern ("SIGCONT", SCM_MAKINUM (SIGCONT));
-#endif
-#ifdef SIGSTOP
-  scm_sysintern ("SIGSTOP", SCM_MAKINUM (SIGSTOP));
-#endif
-#ifdef SIGTSTP
-  scm_sysintern ("SIGTSTP", SCM_MAKINUM (SIGTSTP));
-#endif
-#ifdef SIGTTIN
-  scm_sysintern ("SIGTTIN", SCM_MAKINUM (SIGTTIN));
-#endif
-#ifdef SIGTTOU
-  scm_sysintern ("SIGTTOU", SCM_MAKINUM (SIGTTOU));
-#endif
-#ifdef SIGIO
-  scm_sysintern ("SIGIO", SCM_MAKINUM (SIGIO));
-#endif
-#ifdef SIGPOLL
-  scm_sysintern ("SIGPOLL", SCM_MAKINUM (SIGPOLL));
-#endif
-#ifdef SIGURG
-  scm_sysintern ("SIGURG", SCM_MAKINUM (SIGURG));
-#endif
-#ifdef SIGXCPU
-  scm_sysintern ("SIGXCPU", SCM_MAKINUM (SIGXCPU));
-#endif
-#ifdef SIGXFSZ
-  scm_sysintern ("SIGXFSZ", SCM_MAKINUM (SIGXFSZ));
-#endif
-#ifdef SIGVTALRM
-  scm_sysintern ("SIGVTALRM", SCM_MAKINUM (SIGVTALRM));
-#endif
-#ifdef SIGPROF
-  scm_sysintern ("SIGPROF", SCM_MAKINUM (SIGPROF));
-#endif
-#ifdef SIGWINCH
-  scm_sysintern ("SIGWINCH", SCM_MAKINUM (SIGWINCH));
-#endif
-#ifdef SIGLOST
-  scm_sysintern ("SIGLOST", SCM_MAKINUM (SIGLOST));
-#endif
-#ifdef SIGPWR
-  scm_sysintern ("SIGPWR", SCM_MAKINUM (SIGPWR));
-#endif
   /* access() symbols.  */
   scm_sysintern ("R_OK", SCM_MAKINUM (R_OK));
   scm_sysintern ("W_OK", SCM_MAKINUM (W_OK));
@@ -1457,5 +1203,6 @@ scm_init_posix ()
 #ifdef LC_ALL
   scm_sysintern ("LC_ALL", SCM_MAKINUM (LC_ALL));
 #endif
+#include "cpp_sig_symbols.c"
 #include "posix.x"
 }
