@@ -2,7 +2,7 @@
 
 #ifndef POSIXH
 #define POSIXH
-/*	Copyright (C) 1995 Free Software Foundation, Inc.
+/*	Copyright (C) 1995, 1996, 1997 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -65,6 +65,9 @@ extern SCM scm_getgrgid SCM_P ((SCM name));
 extern SCM scm_setgrent SCM_P ((SCM arg));
 extern SCM scm_kill SCM_P ((SCM pid, SCM sig));
 extern SCM scm_waitpid SCM_P ((SCM pid, SCM options));
+extern SCM scm_status_exit_val SCM_P ((SCM status));
+extern SCM scm_status_term_sig SCM_P ((SCM status));
+extern SCM scm_status_stop_sig SCM_P ((SCM status));
 extern SCM scm_getppid SCM_P ((void));
 extern SCM scm_getuid SCM_P ((void));
 extern SCM scm_getgid SCM_P ((void));
@@ -81,19 +84,13 @@ extern SCM scm_fork SCM_P ((void));
 extern SCM scm_uname SCM_P ((void));
 extern SCM scm_environ SCM_P ((SCM env));
 extern SCM scm_open_pipe SCM_P ((SCM pipestr, SCM modes));
-extern SCM scm_open_input_pipe SCM_P ((SCM pipestr));
-extern SCM scm_open_output_pipe SCM_P ((SCM pipestr));
+extern SCM scm_close_pipe SCM_P ((SCM port));
 extern SCM scm_utime SCM_P ((SCM pathname, SCM actime, SCM modtime));
 extern SCM scm_access SCM_P ((SCM path, SCM how));
 extern SCM scm_getpid SCM_P ((void));
 extern SCM scm_putenv SCM_P ((SCM str));
-extern SCM scm_read_line SCM_P ((SCM port, SCM include_terminator));
-extern SCM scm_read_line_x SCM_P ((SCM str, SCM port));
-extern SCM scm_write_line SCM_P ((SCM obj, SCM port));
 extern SCM scm_setlocale SCM_P ((SCM category, SCM locale));
-extern SCM scm_strftime SCM_P ((SCM format, SCM stime));
-extern SCM scm_strptime SCM_P ((SCM format, SCM string));
-extern SCM scm_mknod SCM_P ((SCM path, SCM mode, SCM dev));
+extern SCM scm_mknod SCM_P ((SCM path, SCM type, SCM perms, SCM dev));
 extern SCM scm_nice SCM_P ((SCM incr));
 extern SCM scm_sync SCM_P ((void));
 extern void scm_init_posix SCM_P ((void));
