@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -157,7 +157,7 @@ scm_print_options (setting)
 
 static SCM print_state_pool;
 
-#if 1 /* Used for debugging purposes */
+#ifdef GUILE_DEBUG /* Used for debugging purposes */
 SCM_PROC(s_current_pstate, "current-pstate", 0, 0, 0, scm_current_pstate);
 
 SCM
@@ -494,18 +494,29 @@ taloop:
 	  ENTER_NESTED_DATA (pstate, exp, circref);
 	  scm_gen_puts (scm_regular_string, "#(", port);
 	common_vector_printer:
-	  for (i = 0; i + 1 < SCM_LENGTH (exp); ++i)
-	    {
-	      /* CHECK_INTS; */
-	      scm_iprin1 (SCM_VELTS (exp)[i], port, pstate);
-	      scm_gen_putc (' ', port);
-	    }
-	  if (i < SCM_LENGTH (exp))
-	    {
-	      /* CHECK_INTS; */
-	      scm_iprin1 (SCM_VELTS (exp)[i], port, pstate);
-	    }
-	  scm_gen_putc (')', port);
+	  {
+	    int last = SCM_LENGTH (exp) - 1;
+	    int cutp = 0;
+	    if (pstate->fancyp && SCM_LENGTH (exp) > pstate->length)
+	      {
+		last = pstate->length - 1;
+		cutp = 1;
+	      }
+	    for (i = 0; i < last; ++i)
+	      {
+		/* CHECK_INTS; */
+		scm_iprin1 (SCM_VELTS (exp)[i], port, pstate);
+		scm_gen_putc (' ', port);
+	      }
+	    if (i == last)
+	      {
+		/* CHECK_INTS; */
+		scm_iprin1 (SCM_VELTS (exp)[i], port, pstate);
+	      }
+	    if (cutp)
+	      scm_gen_puts (scm_regular_string, " ...", port);
+	    scm_gen_putc (')', port);
+	  }
 	  EXIT_NESTED_DATA (pstate);
 	  break;
 	case scm_tc7_bvect:
@@ -650,7 +661,7 @@ void
 scm_iprlist (hdr, exp, tlr, port, pstate)
      char *hdr;
      SCM exp;
-     char tlr;
+     int tlr;
      SCM port;
      scm_print_state *pstate;
 {
