@@ -64,8 +64,32 @@ typedef struct
   int (*vproc) ();
 } ra_iproc;
 
-static ra_iproc ra_rpsubrs[];
-static ra_iproc ra_asubrs[];
+
+/* These tables are a kluge that will not scale well when more
+ * vectorized subrs are added.  It is tempting to steal some bits from
+ * the SCM_CAR of all subrs (like those selected by SCM_SMOBNUM) to hold an
+ * offset into a table of vectorized subrs.  
+ */
+
+static ra_iproc ra_rpsubrs[] =
+{
+  {"=", SCM_UNDEFINED, scm_ra_eqp},
+  {"<", SCM_UNDEFINED, scm_ra_lessp},
+  {"<=", SCM_UNDEFINED, scm_ra_leqp},
+  {">", SCM_UNDEFINED, scm_ra_grp},
+  {">=", SCM_UNDEFINED, scm_ra_greqp},
+  {0, 0, 0}
+};
+
+static ra_iproc ra_asubrs[] =
+{
+  {"+", SCM_UNDEFINED, scm_ra_sum},
+  {"-", SCM_UNDEFINED, scm_ra_difference},
+  {"*", SCM_UNDEFINED, scm_ra_product},
+  {"/", SCM_UNDEFINED, scm_ra_divide},
+  {0, 0, 0}
+};
+
 
 #define BVE_REF(a, i) ((SCM_VELTS(a)[(i)/SCM_LONG_BIT] & (1L<<((i)%SCM_LONG_BIT))) ? 1 : 0)
 #define BVE_SET(a, i) (SCM_VELTS(a)[(i)/SCM_LONG_BIT] |= (1L<<((i)%SCM_LONG_BIT)))
@@ -136,6 +160,7 @@ scm_ra_matchp (ra0, ras)
 	return 0;
       case scm_tc7_vector:
       case scm_tc7_string:
+      case scm_tc7_byvect:
       case scm_tc7_bvect:
       case scm_tc7_uvect:
       case scm_tc7_ivect:
@@ -167,6 +192,7 @@ scm_ra_matchp (ra0, ras)
 	      return 0;
 	    case scm_tc7_vector:
 	    case scm_tc7_string:
+	    case scm_tc7_byvect:
 	    case scm_tc7_bvect:
 	    case scm_tc7_uvect:
 	    case scm_tc7_ivect:
@@ -373,7 +399,17 @@ scm_ramapc (cproc, data, ra0, lra, what)
 }
 
 
-static char s_array_fill_x[];
+SCM_PROC(s_array_fill_x, "array-fill!", 2, 0, 0, scm_array_fill_x);
+
+SCM 
+scm_array_fill_x (ra, fill)
+     SCM ra;
+     SCM fill;
+{
+  scm_ramapc (scm_array_fill_int, fill, ra, SCM_EOL, s_array_fill_x);
+  return SCM_UNSPECIFIED;
+}
+
 
 int 
 scm_array_fill_int (ra, fill, ignore)
@@ -401,6 +437,15 @@ scm_array_fill_int (ra, fill, ignore)
 	for (i = base; n--; i += inc)
 	  SCM_CHARS (ra)[i] = SCM_ICHR (fill);
 	break;
+      case scm_tc7_byvect:
+	if (SCM_ICHRP (fill))
+	  fill = SCM_MAKINUM ((char) SCM_ICHR (fill));
+	SCM_ASRTGO (SCM_INUMP (fill)
+		    && -128 <= SCM_INUM (fill) && SCM_INUM (fill) < 128,
+		    badarg2);
+	for (i = base; n--; i += inc)
+	  SCM_CHARS (ra)[i] = SCM_INUM (fill);
+	break;
       case scm_tc7_bvect:
 	{
 	  long *ve = (long *) SCM_VELTS (ra);
@@ -491,17 +536,6 @@ scm_array_fill_int (ra, fill, ignore)
   return 1;
 }
 
-SCM_PROC(s_array_fill_x, "array-fill!", 2, 0, 0, scm_array_fill_x);
-
-SCM 
-scm_array_fill_x (ra, fill)
-     SCM ra;
-     SCM fill;
-{
-  scm_ramapc (scm_array_fill_int, fill, ra, SCM_EOL, s_array_fill_x);
-  return SCM_UNSPECIFIED;
-}
-
 
 
 
@@ -529,6 +563,7 @@ racp (src, dst)
 	  scm_array_set_x (dst, scm_cvref (src, i_s, SCM_UNDEFINED), SCM_MAKINUM (i_d));
 	break;
       case scm_tc7_string:
+      case scm_tc7_byvect:
 	if (scm_tc7_string != SCM_TYP7 (dst))
 	  goto gencase;
 	for (; n-- > 0; i_s += inc_s, i_d += inc_d)
@@ -1793,6 +1828,7 @@ scm_array_index_map_x (ra, proc)
 	  return SCM_UNSPECIFIED;
 	}
       case scm_tc7_string:
+      case scm_tc7_byvect:
       case scm_tc7_bvect:
       case scm_tc7_uvect:
       case scm_tc7_ivect:
@@ -1892,6 +1928,7 @@ raeql_1 (ra0, as_equal, ra1)
 	  }
 	return 1;
       case scm_tc7_string:
+      case scm_tc7_byvect:
 	{
 	  char *v0 = SCM_CHARS (ra0) + i0;
 	  char *v1 = SCM_CHARS (ra1) + i1;
@@ -2044,6 +2081,7 @@ scm_array_equal_p (ra0, ra1)
 	goto callequal;
       case scm_tc7_bvect:
       case scm_tc7_string:
+      case scm_tc7_byvect:
       case scm_tc7_uvect:
       case scm_tc7_ivect:
       case scm_tc7_fvect:
@@ -2062,6 +2100,7 @@ scm_array_equal_p (ra0, ra1)
 	goto callequal;
       case scm_tc7_bvect:
       case scm_tc7_string:
+      case scm_tc7_byvect:
       case scm_tc7_uvect:
       case scm_tc7_ivect:
       case scm_tc7_fvect:
@@ -2078,32 +2117,6 @@ scm_array_equal_p (ra0, ra1)
 
 
 
-
-/* These tables are a kluge that will not scale well when more
- * vectorized subrs are added.  It is tempting to steal some bits from
- * the SCM_CAR of all subrs (like those selected by SCM_SMOBNUM) to hold an
- * offset into a table of vectorized subrs.  
- */
-
-static ra_iproc ra_rpsubrs[] =
-{
-  {"=", SCM_UNDEFINED, scm_ra_eqp},
-  {"<", SCM_UNDEFINED, scm_ra_lessp},
-  {"<=", SCM_UNDEFINED, scm_ra_leqp},
-  {">", SCM_UNDEFINED, scm_ra_grp},
-  {">=", SCM_UNDEFINED, scm_ra_greqp},
-  {0, 0, 0}
-};
-
-static ra_iproc ra_asubrs[] =
-{
-  {"+", SCM_UNDEFINED, scm_ra_sum},
-  {"-", SCM_UNDEFINED, scm_ra_difference},
-  {"*", SCM_UNDEFINED, scm_ra_product},
-  {"/", SCM_UNDEFINED, scm_ra_divide},
-  {0, 0, 0}
-};
-
 static void
 init_raprocs (subra)
      ra_iproc *subra;
