@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -58,15 +58,17 @@
 
 
 
-#define default_case_i 0
-
-
+SCM_SYMBOL (scm_keyword_prefix, "prefix");
 
 scm_option scm_read_opts[] = {
   { SCM_OPTION_BOOLEAN, "copy", 0,
     "Copy source code expressions." },
   { SCM_OPTION_BOOLEAN, "positions", 0,
-    "Record positions of source code expressions." }
+    "Record positions of source code expressions." },
+  { SCM_OPTION_BOOLEAN, "case-insensitive", 0,
+    "Convert symbols to lower case."},
+  { SCM_OPTION_SCM, "keywords", SCM_BOOL_F,
+    "Style of keyword recognition: #f or 'prefix"}
 };
 
 SCM_PROC (s_read_options, "read-options-interface", 0, 1, 0, scm_read_options);
@@ -84,17 +86,17 @@ scm_read_options (setting)
   return ans;
 }
 
-SCM_PROC (s_read, "read", 0, 3, 0, scm_read);
+/* An association list mapping extra hash characters to procedures.  */
+static SCM *scm_read_hash_procedures;
+
+SCM_PROC (s_read, "read", 0, 1, 0, scm_read);
 
 SCM 
-scm_read (port, case_insensitive_p, sharp)
+scm_read (port)
      SCM port;
-     SCM case_insensitive_p;
-     SCM sharp;
 {
   int c;
   SCM tok_buf, copy;
-  int case_i;
 
   if (SCM_UNBNDP (port))
     port = scm_cur_inp;
@@ -104,20 +106,13 @@ scm_read (port, case_insensitive_p, sharp)
 		SCM_ARG1,
 		s_read);
 
-  case_i = (SCM_UNBNDP (case_insensitive_p)
-	    ? default_case_i
-	    : (case_insensitive_p == SCM_BOOL_F));
-  
-  if (SCM_UNBNDP (sharp))
-    sharp = SCM_BOOL_F;
-
   c = scm_flush_ws (port, (char *) NULL);
   if (EOF == c)
     return SCM_EOF_VAL;
   scm_gen_ungetc (c, port);
 
   tok_buf = scm_makstr (30L, 0);
-  return scm_lreadr (&tok_buf, port, case_i, sharp, &copy);
+  return scm_lreadr (&tok_buf, port, &copy);
 }
 
 
@@ -244,13 +239,16 @@ recsexpr (obj, line, column, filename)
 
 
 /* Consume an SCSH-style block comment.  Assume that we've already
-   read the initial `#!', and eat characters until the matching `!#'.  */
+   read the initial `#!', and eat characters until we get a
+   newline/exclamation-point/sharp-sign/newline sequence.  */
 
 static void
 skip_scsh_block_comment (port)
      SCM port;
 {
-  char last_c = '\0';
+  /* Is this portable?  Dear God, spare me from the non-eight-bit
+     characters.  But is it tasteful?  */
+  long history = 0;
 
   for (;;)
     {
@@ -259,28 +257,29 @@ skip_scsh_block_comment (port)
       if (c == EOF)
 	scm_wta (SCM_UNDEFINED,
 		 "unterminated `#! ... !#' comment", "read");
-      else if (c == '#' && last_c == '!')
-	return;
+      history = ((history << 8) | (c & 0xff)) & 0xffffffff;
 
-      last_c = c;
+      /* Were the last four characters read "\n!#\n"?  */
+      if (history == (('\n' << 24) | ('!' << 16) | ('#' << 8) | '\n'))
+	return;
     }
 }
 
+static SCM
+scm_get_hash_procedure SCM_P ((int c));
 
 static char s_list[]="list";
 
 SCM 
-scm_lreadr (tok_buf, port, case_i, sharp, copy)
+scm_lreadr (tok_buf, port, copy)
      SCM *tok_buf;
      SCM port;
-     int case_i;
-     SCM sharp;
      SCM *copy;
 {
   int c;
   scm_sizet j;
   SCM p;
-
+				  
 tryagain:
   c = scm_flush_ws (port, s_read);
 tryagain_no_flush_ws:
@@ -291,8 +290,8 @@ tryagain_no_flush_ws:
 
     case '(':
       return SCM_RECORD_POSITIONS_P
-	     ? scm_lreadrecparen (tok_buf, port, s_list, case_i, sharp, copy)
-	     : scm_lreadparen (tok_buf, port, s_list, case_i, sharp, copy);
+	     ? scm_lreadrecparen (tok_buf, port, s_list, copy)
+	     : scm_lreadparen (tok_buf, port, s_list, copy);
     case ')':
       scm_wta (SCM_UNDEFINED, "unexpected \")\"", "read");
       goto tryagain;
@@ -314,7 +313,7 @@ tryagain_no_flush_ws:
 	}
     recquote:
       p = scm_cons2 (p,
-		     scm_lreadr (tok_buf, port, case_i, sharp, copy),
+		     scm_lreadr (tok_buf, port, copy),
 		     SCM_EOL);
       if (SCM_RECORD_POSITIONS_P)
 	scm_whash_insert (scm_source_whash,
@@ -334,7 +333,7 @@ tryagain_no_flush_ws:
       switch (c)
 	{
 	case '(':
-	  p = scm_lreadparen (tok_buf, port, "vector", case_i, sharp, copy);
+	  p = scm_lreadparen (tok_buf, port, "vector", copy);
 	  return SCM_NULLP (p) ? scm_nullvect : scm_vector (p);
 
 	case 't':
@@ -369,7 +368,7 @@ tryagain_no_flush_ws:
 	  goto tryagain_no_flush_ws;
 
 	case '*':
-	  j = scm_read_token (c, tok_buf, port, case_i, 0);
+	  j = scm_read_token (c, tok_buf, port, 0);
 	  p = scm_istr2bve (SCM_CHARS (*tok_buf) + 1, (long) (j - 1));
 	  if (SCM_NFALSEP (p))
 	    return p;
@@ -377,7 +376,7 @@ tryagain_no_flush_ws:
 	    goto unkshrp;
 
 	case '{':
-	  j = scm_read_token (c, tok_buf, port, case_i, 1);
+	  j = scm_read_token (c, tok_buf, port, 1);
 	  p = scm_intern (SCM_CHARS (*tok_buf), j);
 	  if (SCM_PORT_REPRESENTATION (port) != scm_regular_port)
 	    scm_set_symbol_multi_byte_x (SCM_CAR (p), SCM_BOOL_T);
@@ -385,7 +384,7 @@ tryagain_no_flush_ws:
 
 	case '\\':
 	  c = scm_gen_getc (port);
-	  j = scm_read_token (c, tok_buf, port, case_i, 0);
+	  j = scm_read_token (c, tok_buf, port, 0);
 	  if (j == 1)
 	    return SCM_MAKICHR (c);
 	  if (c >= '0' && c < '8')
@@ -403,23 +402,30 @@ tryagain_no_flush_ws:
 
 	default:
 	callshrp:
-	  if (SCM_NIMP (sharp))
-	    {
-	      int line = SCM_LINUM (port);
-	      int column = SCM_COL (port) - 2;
-	      SCM got;
-	      got = scm_apply (sharp,
-			       SCM_MAKICHR (c),
-			       scm_acons (port, SCM_EOL, SCM_EOL));
-	      if (SCM_UNSPECIFIED == got)
-		goto unkshrp;
-	      if (SCM_RECORD_POSITIONS_P)
-		return *copy = recsexpr (got, line, column,
-					 SCM_FILENAME (port));
-	      else
-		return got;
-	    }
-	unkshrp:scm_wta ((SCM) SCM_MAKICHR (c), "unknown # object", "");
+	  {
+	    SCM sharp = scm_get_hash_procedure (c);
+
+	    if (SCM_NIMP (sharp))
+	      {
+		int line = SCM_LINUM (port);
+		int column = SCM_COL (port) - 2;
+		SCM got;
+
+		got = scm_apply (sharp,
+				 SCM_MAKICHR (c),
+				 scm_acons (port, SCM_EOL, SCM_EOL));
+		if (SCM_UNSPECIFIED == got)
+		  goto unkshrp;
+		if (SCM_RECORD_POSITIONS_P)
+		  return *copy = recsexpr (got, line, column,
+					   SCM_FILENAME (port));
+		else
+		  return got;
+	      }
+	  }
+	unkshrp:
+	  scm_misc_error (s_read, "Unknown # object: %S",
+			  scm_listify (SCM_MAKICHR (c), SCM_UNDEFINED));
 	}
 
     case '"':
@@ -492,7 +498,7 @@ tryagain_no_flush_ws:
     case '-':
     case '+':
     num:
-      j = scm_read_token (c, tok_buf, port, case_i, 0);
+      j = scm_read_token (c, tok_buf, port, 0);
       p = scm_istring2number (SCM_CHARS (*tok_buf), (long) j, 10L);
       if (SCM_NFALSEP (p))
 	return p;
@@ -509,14 +515,17 @@ tryagain_no_flush_ws:
       goto tok;
 
     case ':':
-      j = scm_read_token ('-', tok_buf, port, case_i, 0);
-      p = scm_intern (SCM_CHARS (*tok_buf), j);
-      if (SCM_PORT_REPRESENTATION (port) != scm_regular_port)
-	scm_set_symbol_multi_byte_x (SCM_CAR (p), SCM_BOOL_T);
-      return scm_make_keyword_from_dash_symbol (SCM_CAR (p));
-
+      if (SCM_KEYWORD_STYLE == scm_keyword_prefix)
+	{
+	  j = scm_read_token ('-', tok_buf, port, 0);
+	  p = scm_intern (SCM_CHARS (*tok_buf), j);
+	  if (SCM_PORT_REPRESENTATION (port) != scm_regular_port)
+	    scm_set_symbol_multi_byte_x (SCM_CAR (p), SCM_BOOL_T);
+	  return scm_make_keyword_from_dash_symbol (SCM_CAR (p));
+	}
+      /* fallthrough */
     default:
-      j = scm_read_token (c, tok_buf, port, case_i, 0);
+      j = scm_read_token (c, tok_buf, port, 0);
       /* fallthrough */
 
     tok:
@@ -532,18 +541,17 @@ _Pragma ("noopt");		/* # pragma _CRI noopt */
 #endif
 
 scm_sizet 
-scm_read_token (ic, tok_buf, port, case_i, weird)
+scm_read_token (ic, tok_buf, port, weird)
      int ic;
      SCM *tok_buf;
      SCM port;
-     int case_i;
      int weird;
 {
   register scm_sizet j;
   register int c;
   register char *p;
 
-  c = ic;
+  c = (SCM_CASE_INSENSITIVE_P ? scm_downcase(ic) : ic);
   p = SCM_CHARS (*tok_buf);
 
   if (weird)
@@ -621,7 +629,7 @@ scm_read_token (ic, tok_buf, port, case_i, weird)
 	default:
 	default_case:
 	  {
-	    c = (case_i ? scm_downcase(c) : c);
+	    c = (SCM_CASE_INSENSITIVE_P ? scm_downcase(c) : c);
 	    if (SCM_PORT_REPRESENTATION(port) == scm_regular_port)
 	      {
 		p[j] = c;
@@ -647,12 +655,10 @@ _Pragma ("opt");		/* # pragma _CRI opt */
 #endif
 
 SCM 
-scm_lreadparen (tok_buf, port, name, case_i, sharp, copy)
+scm_lreadparen (tok_buf, port, name, copy)
      SCM *tok_buf;
      SCM port;
      char *name;
-     int case_i;
-     SCM sharp;
      SCM *copy;
 {
   SCM tmp;
@@ -664,9 +670,9 @@ scm_lreadparen (tok_buf, port, name, case_i, sharp, copy)
   if (')' == c)
     return SCM_EOL;
   scm_gen_ungetc (c, port);
-  if (scm_i_dot == (tmp = scm_lreadr (tok_buf, port, case_i, sharp, copy)))
+  if (scm_i_dot == (tmp = scm_lreadr (tok_buf, port, copy)))
     {
-      ans = scm_lreadr (tok_buf, port, case_i, sharp, copy);
+      ans = scm_lreadr (tok_buf, port, copy);
     closeit:
       if (')' != (c = scm_flush_ws (port, name)))
 	scm_wta (SCM_UNDEFINED, "missing close paren", "");
@@ -676,9 +682,9 @@ scm_lreadparen (tok_buf, port, name, case_i, sharp, copy)
   while (')' != (c = scm_flush_ws (port, name)))
     {
       scm_gen_ungetc (c, port);
-      if (scm_i_dot == (tmp = scm_lreadr (tok_buf, port, case_i, sharp, copy)))
+      if (scm_i_dot == (tmp = scm_lreadr (tok_buf, port, copy)))
 	{
-	  SCM_SETCDR (tl, scm_lreadr (tok_buf, port, case_i, sharp, copy));
+	  SCM_SETCDR (tl, scm_lreadr (tok_buf, port, copy));
 	  goto closeit;
 	}
       SCM_SETCDR (tl, scm_cons (tmp, SCM_EOL));
@@ -689,12 +695,10 @@ scm_lreadparen (tok_buf, port, name, case_i, sharp, copy)
 
 
 SCM 
-scm_lreadrecparen (tok_buf, port, name, case_i, sharp, copy)
+scm_lreadrecparen (tok_buf, port, name, copy)
      SCM *tok_buf;
      SCM port;
      char *name;
-     int case_i;
-     SCM sharp;
      SCM *copy;
 {
   register int c;
@@ -709,9 +713,9 @@ scm_lreadrecparen (tok_buf, port, name, case_i, sharp, copy)
   if (')' == c)
     return SCM_EOL;
   scm_gen_ungetc (c, port);
-  if (scm_i_dot == (tmp = scm_lreadr (tok_buf, port, case_i, sharp, copy)))
+  if (scm_i_dot == (tmp = scm_lreadr (tok_buf, port, copy)))
     {
-      ans = scm_lreadr (tok_buf, port, case_i, sharp, copy);
+      ans = scm_lreadr (tok_buf, port, copy);
       if (')' != (c = scm_flush_ws (port, name)))
 	scm_wta (SCM_UNDEFINED, "missing close paren", "");
       return ans;
@@ -726,9 +730,9 @@ scm_lreadrecparen (tok_buf, port, name, case_i, sharp, copy)
   while (')' != (c = scm_flush_ws (port, name)))
     {
       scm_gen_ungetc (c, port);
-      if (scm_i_dot == (tmp = scm_lreadr (tok_buf, port, case_i, sharp, copy)))
+      if (scm_i_dot == (tmp = scm_lreadr (tok_buf, port, copy)))
 	{
-	  SCM_SETCDR (tl, tmp = scm_lreadr (tok_buf, port, case_i, sharp, copy));
+	  SCM_SETCDR (tl, tmp = scm_lreadr (tok_buf, port, copy));
 	  if (SCM_COPY_SOURCE_P)
 	    SCM_SETCDR (tl2, scm_cons (SCM_NIMP (tmp) && SCM_CONSP (tmp)
 				       ? *copy
@@ -761,11 +765,89 @@ exit:
 
 
 
+/* Manipulate the read-hash-procedures alist.  This could be written in
+   Scheme, but maybe it will also be used by C code during initialisation.  */
+SCM_PROC (s_read_hash_extend, "read-hash-extend", 2, 0, 0, scm_read_hash_extend);
+SCM
+scm_read_hash_extend (chr, proc)
+     SCM chr;
+     SCM proc;
+{
+  SCM this;
+  SCM prev;
 
+  SCM_ASSERT (SCM_ICHRP(chr), chr, SCM_ARG1, s_read_hash_extend);
+  SCM_ASSERT (SCM_FALSEP (proc) || SCM_NIMP(proc), proc, SCM_ARG2,
+	      s_read_hash_extend);
+
+  /* Check if chr is already in the alist.  */
+  this = *scm_read_hash_procedures;
+  prev = SCM_BOOL_F;
+  while (1)
+    {
+      if (SCM_NULLP (this))
+	{
+	  /* not found, so add it to the beginning.  */
+	  if (SCM_NFALSEP (proc))
+	    {
+	      *scm_read_hash_procedures = 
+		scm_cons (scm_cons (chr, proc), *scm_read_hash_procedures);
+	    }
+	  break;
+	}
+      if (chr == SCM_CAAR (this))
+	{
+	  /* already in the alist.  */
+	  if (SCM_FALSEP (proc))
+	    {
+	      /* remove it.  */
+	      if (prev == SCM_BOOL_F)
+		{
+		  *scm_read_hash_procedures =
+		    SCM_CDR (*scm_read_hash_procedures);
+		}
+	      else
+		scm_set_cdr_x (prev, SCM_CDR (this));
+	    }
+	  else
+	    {
+	      /* replace it.  */
+	      scm_set_cdr_x (SCM_CAR (this), proc);
+	    }
+	  break;
+	}
+      prev = this;
+      this = SCM_CDR (this);
+    }
+
+  return SCM_UNSPECIFIED;
+}
+
+/* Recover the read-hash procedure corresponding to char c.  */
+static SCM
+scm_get_hash_procedure (c)
+     int c;
+{
+  SCM rest = *scm_read_hash_procedures;
+
+  while (1)
+    {
+      if (SCM_NULLP (rest))
+	return SCM_BOOL_F;
+  
+      if (SCM_ICHR (SCM_CAAR (rest)) == c)
+	return SCM_CDAR (rest);
+     
+      rest = SCM_CDR (rest);
+    }
+}
 
 void
 scm_init_read ()
 {
+  scm_read_hash_procedures =
+    SCM_CDRLOC (scm_sysintern ("read-hash-procedures", SCM_EOL));
+
   scm_init_opts (scm_read_options, scm_read_opts, SCM_N_READ_OPTIONS);
 #include "read.x"
 }
