@@ -2,7 +2,7 @@
 
 #ifndef SCMSIGSH
 #define SCMSIGSH
-/*	Copyright (C) 1995 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,38 +43,18 @@
  * If you do not wish that, delete this exception notice.  
  */
 
-
+#include "libguile/__scm.h"
 
 
-#ifdef __STDC__
-extern SCM scm_alarm (SCM i);
-extern SCM scm_pause (void);
-extern SCM scm_sleep (SCM i);
-extern SCM scm_raise(SCM sig);
-extern void scm_tick_signal (void);
-extern SCM scm_ticks (SCM i);
-extern void scm_init_signals (void);
-extern void scm_ignore_signals (void);
-extern void scm_unignore_signals (void);
-extern void scm_restore_signals (void);
-extern void scm_init_scmsigs (void);
-
-#else /* STDC */
-extern SCM scm_alarm ();
-extern SCM scm_pause ();
-extern SCM scm_sleep ();
-extern SCM scm_raise();
-extern void scm_tick_signal ();
-extern SCM scm_ticks ();
-extern void scm_init_signals ();
-extern void scm_ignore_signals ();
-extern void scm_unignore_signals ();
-extern void scm_restore_signals ();
-extern void scm_init_scmsigs ();
-
-#endif /* STDC */
-
-
 
+extern SCM scm_alarm SCM_P ((SCM i));
+extern SCM scm_pause SCM_P ((void));
+extern SCM scm_sleep SCM_P ((SCM i));
+extern SCM scm_raise SCM_P ((SCM sig));
+extern void scm_init_signals SCM_P ((void));
+extern void scm_ignore_signals SCM_P ((void));
+extern void scm_unignore_signals SCM_P ((void));
+extern SCM scm_restore_signals SCM_P ((void));
+extern void scm_init_scmsigs SCM_P ((void));
 
 #endif  /* SCMSIGSH */
