@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,6 +47,7 @@
 #include "variable.h"
 #include "alist.h"
 #include "mbstrings.h"
+#include "weaks.h"
 
 #include "symbols.h"
 
@@ -144,7 +145,7 @@ scm_sym2vcell (sym, thunk, definep)
 	  z = SCM_CAR (lsym);
 	  if (SCM_CAR (z) == sym)
 	    {
-	      if (definep)
+	      if (SCM_NFALSEP (definep))
 		{
 		  /* Move handle from scm_weak_symhash to scm_symhash. */
 		  *lsymp = SCM_CDR (lsym);
@@ -356,19 +357,16 @@ scm_intern0 (name)
 }
 
 
-/* Intern the symbol named NAME in scm_symhash, and give it the value VAL.
-   NAME is null-terminated.  */
+/* Intern the symbol named NAME in scm_symhash, NAME is null-terminated.  */
 SCM 
-scm_sysintern (name, val)
+scm_sysintern0_no_module_lookup (name)
      char *name;
-     SCM val;
 {
   SCM easy_answer;
   SCM_DEFER_INTS;
   easy_answer = scm_intern_obarray_soft (name, strlen (name), scm_symhash, 1);
   if (SCM_NIMP (easy_answer))
     {
-      SCM_SETCDR (easy_answer, val);
       SCM_ALLOW_INTS;
       return easy_answer;
     }
@@ -381,7 +379,7 @@ scm_sysintern (name, val)
       SCM_NEWCELL (lsym);
       SCM_SETLENGTH (lsym, (long) len, scm_tc7_ssymbol);
       SCM_SETCHARS (lsym, name);
-      lsym = scm_cons (lsym, val);
+      lsym = scm_cons (lsym, SCM_UNDEFINED);
       SCM_VELTS (scm_symhash)[scm_hash] = scm_cons (lsym, SCM_VELTS (scm_symhash)[scm_hash]);
       SCM_ALLOW_INTS;
       return lsym;
@@ -389,6 +387,60 @@ scm_sysintern (name, val)
 }
 
 
+/* Is it safe to access SCM_TOP_LEVEL_LOOKUP_CLOSURE_VAR?
+ */
+int scm_can_use_top_level_lookup_closure_var;
+
+/* Intern the symbol named NAME in scm_symhash, and give it the value
+   VAL.  NAME is null-terminated.  Use the current top_level lookup
+   closure to give NAME its value.
+ */
+SCM
+scm_sysintern (name, val)
+     char *name;
+     SCM val;
+{
+  SCM vcell = scm_sysintern0 (name);
+  SCM_SETCDR (vcell, val);
+  return vcell;
+}
+
+SCM
+scm_sysintern0 (name)
+     char *name;
+{
+  SCM lookup_proc;
+  if (scm_can_use_top_level_lookup_closure_var && 
+      SCM_NIMP (lookup_proc = SCM_CDR (scm_top_level_lookup_closure_var)))
+    {
+      SCM sym = SCM_CAR (scm_intern0 (name));
+      SCM vcell = scm_sym2vcell (sym, lookup_proc, SCM_BOOL_T);
+      if (vcell == SCM_BOOL_F)
+	  scm_misc_error ("sysintern", "can't define variable", sym);
+      return vcell;
+    }
+  else
+    return scm_sysintern0_no_module_lookup (name);
+}
+
+/* Lookup the value of the symbol named by the nul-terminated string
+   NAME in the current module.  */
+SCM
+scm_symbol_value0 (name)
+     char *name;
+{
+  /* This looks silly - we look up the symbol twice.  But it is in
+     fact necessary given the current module system because the module
+     lookup closures are written in scheme which needs real symbols. */
+  SCM symbol = scm_intern_obarray_soft (name, strlen (name), scm_symhash, 0);
+  SCM vcell = scm_sym2vcell (SCM_CAR (symbol),
+			     SCM_CDR (scm_top_level_lookup_closure_var),
+			     SCM_BOOL_F);
+  if (SCM_FALSEP (vcell))
+    return SCM_UNDEFINED;
+  return SCM_CDR (vcell);
+}
+
 SCM_PROC(s_symbol_p, "symbol?", 1, 0, 0, scm_symbol_p);
 
 SCM
@@ -640,6 +692,11 @@ msymbolize (s)
   SCM_SYMBOL_MULTI_BYTE_STRINGP (s) = SCM_BOOL_F;
   SCM_SETCDR (string, SCM_EOL);
   SCM_SETCAR (string, SCM_EOL);
+  SCM_SYMBOL_PROPS (s) = SCM_EOL;
+  /* If it's a tc7_ssymbol, it comes from scm_symhash */
+  SCM_SYMBOL_HASH (s) = scm_strhash (SCM_UCHARS (s),
+				     (scm_sizet) SCM_LENGTH (s),
+				     SCM_LENGTH (scm_symhash));
 }
 
 
@@ -714,10 +771,65 @@ scm_symbol_hash (s)
      SCM s;
 {
   SCM_ASSERT(SCM_NIMP(s) && SCM_SYMBOLP(s), s, SCM_ARG1, s_symbol_hash);
+  if (SCM_TYP7(s) == scm_tc7_ssymbol)
+    msymbolize (s);
   return SCM_MAKINUM ((unsigned long)s ^ SCM_SYMBOL_HASH (s));
 }
 
 
+static void copy_and_prune_obarray SCM_P ((SCM from, SCM to));
+
+static void
+copy_and_prune_obarray (from, to)
+     SCM from;
+     SCM to;
+{
+  int i;
+  int length = SCM_LENGTH (from);
+  for (i = 0; i < length; ++i)
+    {
+      SCM head = SCM_VELTS (from)[i]; /* GC protection */
+      SCM ls = head;
+      SCM res = SCM_EOL;
+      SCM *lloc = &res;
+      while (SCM_NIMP (ls))
+	{
+	  if (!SCM_UNBNDP (SCM_CDAR (ls)))
+	    {
+	      *lloc = scm_cons (SCM_CAR (ls), SCM_EOL);
+	      lloc = SCM_CDRLOC (*lloc);
+	    }
+	  ls = SCM_CDR (ls);
+	}
+      SCM_VELTS (to)[i] = res;
+    }
+}
+
+
+SCM_PROC(s_builtin_bindings, "builtin-bindings", 0, 0, 0, scm_builtin_bindings);
+
+SCM
+scm_builtin_bindings ()
+{
+  int length = SCM_LENGTH (scm_symhash);
+  SCM obarray = scm_make_vector (SCM_MAKINUM (length), SCM_EOL, SCM_UNDEFINED);
+  copy_and_prune_obarray (scm_symhash, obarray);
+  return obarray;
+}
+
+
+SCM_PROC(s_builtin_weak_bindings, "builtin-weak-bindings", 0, 0, 0, scm_builtin_weak_bindings);
+
+SCM
+scm_builtin_weak_bindings ()
+{
+  int length = SCM_LENGTH (scm_weak_symhash);
+  SCM obarray = scm_make_doubly_weak_hash_table (SCM_MAKINUM (length));
+  copy_and_prune_obarray (scm_weak_symhash, obarray);
+  return obarray;
+}
+
+
 
 void
 scm_init_symbols ()
