@@ -2,7 +2,7 @@
 
 #ifndef SYMBOLSH
 #define SYMBOLSH
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -100,6 +100,9 @@ extern int scm_symhash_dim;
 #define SCM_SUBSTR_STR(x) (SCM_CDDR (x))
 #define SCM_SUBSTR_OFFSET(x) (SCM_CADR (x))
 
+#define SCM_COERCE_SUBSTR(x) { if (SCM_SUBSTRP (x)) \
+				 x = scm_makfromstr (SCM_ROCHARS (x), \
+						     SCM_ROLENGTH (x), 0); }
 
 
 
@@ -112,6 +115,8 @@ extern SCM scm_intern_obarray SCM_P ((char *name, scm_sizet len, SCM obarray));
 extern SCM scm_intern SCM_P ((char *name, scm_sizet len));
 extern SCM scm_intern0 SCM_P ((char * name));
 extern SCM scm_sysintern SCM_P ((char *name, SCM val));
+extern SCM scm_sysintern0 SCM_P ((char *name));
+extern SCM scm_symbol_value0 SCM_P ((char *name));
 extern SCM scm_symbol_p SCM_P ((SCM x));
 extern SCM scm_symbol_to_string SCM_P ((SCM s));
 extern SCM scm_string_to_symbol SCM_P ((SCM s));
@@ -129,4 +134,6 @@ extern SCM scm_symbol_pset_x SCM_P ((SCM s, SCM val));
 extern SCM scm_symbol_hash SCM_P ((SCM s));
 extern void scm_init_symbols SCM_P ((void));
 
+extern int scm_can_use_top_level_lookup_closure_var;
+
 #endif  /* SYMBOLSH */
