@@ -275,12 +275,15 @@ typedef long SCM;
 #define SCM_CONSP(x) (!SCM_NCONSP(x))
 
 
-/* ECONSP is historical and, in fact, slightly buggy.
- * There are two places to fix where structures and glocs can be confused.
- * !!!
+/* SCM_ECONSP should be used instead of SCM_CONSP at places where GLOCS
+ * can be expected to occur.
  */
-#define SCM_ECONSP(x) (SCM_CONSP(x) || (1==SCM_TYP3(x)))
-#define SCM_NECONSP(x) (SCM_NCONSP(x) && (1 != SCM_TYP3(x)))
+#define SCM_ECONSP(x) (SCM_CONSP (x) \
+		       || (SCM_TYP3(x) == 1 \
+                           && SCM_CDR (SCM_CAR (x) - 1) != 0))
+#define SCM_NECONSP(x) (SCM_NCONSP(x) \
+			&& (SCM_TYP3(x) != 1 \
+			    || SCM_CDR (SCM_CAR (x) - 1) == 0))
 
 
 
