@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996,1997 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,6 +46,7 @@
 #include "smob.h"
 #include "alist.h"
 #include "eval.h"
+#include "eq.h"
 #include "dynwind.h"
 #include "backtrace.h"
 #ifdef DEBUG_EXTENSIONS
@@ -57,8 +58,7 @@
 #include "throw.h"
 
 
-/* {Catch and Throw} 
- */
+/* the jump buffer data structure */
 static int scm_tc16_jmpbuffer;
 
 #define SCM_JMPBUFP(O) (SCM_TYP16(O) == scm_tc16_jmpbuffer)
@@ -131,6 +131,9 @@ make_jmpbuf ()
   return answer;
 }
 
+
+/* scm_internal_catch (the guts of catch) */
+
 struct jmp_buf_and_retval	/* use only on the stack, in scm_catch */
 {
   jmp_buf buf;			/* must be first */
@@ -162,24 +165,31 @@ struct jmp_buf_and_retval	/* use only on the stack, in scm_catch */
 
    HANDLER is a pointer to a C function to deal with a throw to TAG,
    should one occur.  We call it like this:
-      HANDLER (HANDLER_DATA, TAG, THROW_ARGS)
+      HANDLER (HANDLER_DATA, THROWN_TAG, THROW_ARGS)
    where
       HANDLER_DATA is the HANDLER_DATA argument we recevied; it's the
          same idea as BODY_DATA above.
-      TAG is the tag that the user threw to; usually this is TAG, but
-         it could be something else if TAG was #t (i.e., a catch-all),
-         or the user threw to a jmpbuf.
+      THROWN_TAG is the tag that the user threw to; usually this is
+         TAG, but it could be something else if TAG was #t (i.e., a
+         catch-all), or the user threw to a jmpbuf.
       THROW_ARGS is the list of arguments the user passed to the THROW
          function.
 
-   BODY_DATA is just a pointer we pass through to BODY.
-   HANDLER_DATA is just a pointer we pass through to HANDLER.
-   We don't actually use either of those pointers otherwise ourselves.
-   The idea is that, if our caller wants to communicate something to
-   BODY or HANDLER, it can pass a pointer to it as MUMBLE_DATA, which
-   BODY and HANDLER can then use.  Think of it as a way to make BODY
-   and HANDLER closures, not just functions; MUMBLE_DATA points to the
-   enclosed variables.  */
+   BODY_DATA is just a pointer we pass through to BODY.  HANDLER_DATA
+   is just a pointer we pass through to HANDLER.  We don't actually
+   use either of those pointers otherwise ourselves.  The idea is
+   that, if our caller wants to communicate something to BODY or
+   HANDLER, it can pass a pointer to it as MUMBLE_DATA, which BODY and
+   HANDLER can then use.  Think of it as a way to make BODY and
+   HANDLER closures, not just functions; MUMBLE_DATA points to the
+   enclosed variables.
+
+   Of course, it's up to the caller to make sure that any data a
+   MUMBLE_DATA needs is protected from GC.  A common way to do this is
+   to make MUMBLE_DATA a pointer to data stored in an automatic
+   structure variable; since the collector must scan the stack for
+   references anyway, this assures that any references in MUMBLE_DATA
+   will be found.  */
 
 SCM
 scm_internal_catch (tag, body, body_data, handler, handler_data)
@@ -231,13 +241,113 @@ scm_internal_catch (tag, body, body_data, handler, handler_data)
 }
 
 
+
+/* scm_internal_lazy_catch (the guts of lazy catching) */
+
+/* The smob tag for lazy_catch smobs.  */
+static long tc16_lazy_catch;
+
+/* This is the structure we put on the wind list for a lazy catch.  It
+   stores the handler function to call, and the data pointer to pass
+   through to it.  It's not a Scheme closure, but it is a function
+   with data, so the term "closure" is appropriate in its broader
+   sense.
+
+   (We don't need anything like this in the "eager" catch code,
+   because the same C frame runs both the body and the handler.)  */
+struct lazy_catch {
+  scm_catch_handler_t handler;
+  void *handler_data;
+};
+
+/* Strictly speaking, we could just pass a zero for our print
+   function, because we don't need to print them.  They should never
+   appear in normal data structures, only in the wind list.  However,
+   it might be nice for debugging someday... */
+static int
+print_lazy_catch (SCM closure, SCM port, scm_print_state *pstate)
+{
+  struct lazy_catch *c = (struct lazy_catch *) SCM_CDR (closure);
+  char buf[200];
+
+  sprintf (buf, "#<lazy-catch 0x%lx 0x%lx>",
+	   (long) c->handler, (long) c->handler_data);
+  scm_gen_puts (scm_regular_string, buf, port);
+
+  return 1;
+}
+
+static scm_smobfuns lazy_catch_funs = {
+  scm_mark0, scm_free0, print_lazy_catch, 0
+};
+
+
+/* Given a pointer to a lazy catch structure, return a smob for it,
+   suitable for inclusion in the wind list.  ("Ah yes, a Chteau
+   Gollombiere '72, non?").  */
+static SCM
+make_lazy_catch (struct lazy_catch *c)
+{
+  SCM smob;
+
+  SCM_NEWCELL (smob);
+  SCM_SETCDR (smob, c);
+  SCM_SETCAR (smob, tc16_lazy_catch);
+
+  return smob;
+}
+
+#define SCM_LAZY_CATCH_P(obj) \
+  (SCM_NIMP (obj) && (SCM_CAR (obj) == tc16_lazy_catch))
+
+
+/* Exactly like scm_internal_catch, except:
+   - It does not unwind the stack (this is the major difference).
+   - If handler returns, its value is returned from the throw.
+   - BODY always receives #f as its JMPBUF argument (since there's no
+     jmpbuf associated with a lazy catch, because we don't unwind the
+     stack.)  */
+SCM
+scm_internal_lazy_catch (tag, body, body_data, handler, handler_data)
+     SCM tag;
+     scm_catch_body_t body;
+     void *body_data;
+     scm_catch_handler_t handler;
+     void *handler_data;
+{
+  SCM lazy_catch, answer;
+  struct lazy_catch c;
+
+  c.handler = handler;
+  c.handler_data = handler_data;
+  lazy_catch = make_lazy_catch (&c);
+
+  SCM_REDEFER_INTS;
+  scm_dynwinds = scm_acons (tag, lazy_catch, scm_dynwinds);
+  SCM_REALLOW_INTS;
+
+  answer = (*body) (body_data, SCM_BOOL_F);
+
+  SCM_REDEFER_INTS;
+  scm_dynwinds = SCM_CDR (scm_dynwinds);
+  SCM_REALLOW_INTS;
+
+  return answer;
+}
+
+
+
+/* body and handler functions for use with either of the above */
+
 /* This is a body function you can pass to scm_internal_catch if you
    want the body to be like Scheme's `catch' --- a thunk, or a
    function of one argument if the tag is #f.
 
-   DATA contains the Scheme procedure to invoke.  If the tag being
-   caught is #f, then we pass JMPBUF to the body procedure; otherwise,
-   it gets no arguments.  */
+   BODY_DATA is a pointer to a scm_body_thunk_data structure, which
+   contains the Scheme procedure to invoke as the body, and the tag
+   we're catching.  If the tag is #f, then we pass JMPBUF (created by
+   scm_internal_catch) to the body procedure; otherwise, the body gets
+   no arguments.  */
 
 SCM
 scm_body_thunk (body_data, jmpbuf)
@@ -253,11 +363,16 @@ scm_body_thunk (body_data, jmpbuf)
 }
 
 
-/* If the user does a throw to this catch, this function runs a
+/* This is a handler function you can pass to scm_internal_catch if
+   you want the handler to act like Scheme's catch --- call a
+   procedure with the tag and the throw arguments.  
+
+   If the user does a throw to this catch, this function runs a
    handler procedure written in Scheme.  HANDLER_DATA is a pointer to
    an SCM variable holding the Scheme procedure object to invoke.  It
-   ought to be a pointer to an automatic, or the procedure object
-   should be otherwise protected from GC.  */
+   ought to be a pointer to an automatic variable (i.e., one living on
+   the stack), or the procedure object should be otherwise protected
+   from GC.  */
 SCM
 scm_handle_by_proc (handler_data, tag, throw_args)
      void *handler_data;
@@ -270,6 +385,83 @@ scm_handle_by_proc (handler_data, tag, throw_args)
 }
 
 
+/* This is a handler function to use if you want scheme to print a
+   message and die.  Useful for dealing with throws to uncaught keys
+   at the top level.
+
+   At boot time, we establish a catch-all that uses this as its handler.
+   1) If the user wants something different, they can use (catch #t
+   ...) to do what they like.
+   2) Outside the context of a read-eval-print loop, there isn't
+   anything else good to do; libguile should not assume the existence
+   of a read-eval-print loop.
+   3) Given that we shouldn't do anything complex, it's much more
+   robust to do it in C code.
+
+   HANDLER_DATA, if non-zero, is assumed to be a char * pointing to a
+   message header to print; if zero, we use "guile" instead.  That
+   text is followed by a colon, then the message described by ARGS.  */
+
+SCM
+scm_handle_by_message (handler_data, tag, args)
+     void *handler_data;
+     SCM tag;
+     SCM args;
+{
+  char *prog_name = (char *) handler_data;
+  SCM p = scm_def_errp;
+
+  if (SCM_NFALSEP (scm_eq_p (tag, SCM_CAR (scm_intern0 ("quit")))))
+    exit (scm_exit_status (args));
+
+  if (! prog_name)
+    prog_name = "guile";
+
+  scm_gen_puts (scm_regular_string, prog_name, p);
+  scm_gen_puts (scm_regular_string, ": ", p);
+
+  if (scm_ilength (args) >= 3)
+    {
+      SCM message = SCM_CADR (args);
+      SCM parts = SCM_CADDR (args);
+
+      scm_display_error_message (message, parts, p);
+    }
+  else
+    {
+      scm_gen_puts (scm_regular_string, "uncaught throw to ", p);
+      scm_prin1 (tag, p, 0);
+      scm_gen_puts (scm_regular_string, ": ", p);
+      scm_prin1 (args, p, 1);
+      scm_gen_putc ('\n', p);
+    }
+
+  exit (2);
+}
+
+
+/* Derive the an exit status from the arguments to (quit ...).  */
+int
+scm_exit_status (args)
+  SCM args;
+{
+  if (SCM_NNULLP (args))
+    {
+      SCM cqa = SCM_CAR (args);
+      
+      if (SCM_INUMP (cqa))
+	return (SCM_INUM (cqa));
+      else if (SCM_FALSEP (cqa))
+	return 1;
+    }
+  return 0;
+}
+	
+
+
+
+/* the Scheme-visible CATCH and LAZY-CATCH functions */
+
 SCM_PROC(s_catch, "catch", 3, 0, 0, scm_catch);
 SCM
 scm_catch (tag, thunk, handler)
@@ -297,6 +489,7 @@ scm_catch (tag, thunk, handler)
 			     scm_handle_by_proc, &handler);
 }
 
+
 SCM_PROC(s_lazy_catch, "lazy-catch", 3, 0, 0, scm_lazy_catch);
 SCM
 scm_lazy_catch (tag, thunk, handler)
@@ -304,70 +497,41 @@ scm_lazy_catch (tag, thunk, handler)
      SCM thunk;
      SCM handler;
 {
-  SCM answer;
+  struct scm_body_thunk_data c;
+
   SCM_ASSERT ((SCM_NIMP(tag) && SCM_SYMBOLP(tag))
 	      || (tag == SCM_BOOL_T),
 	      tag, SCM_ARG1, s_lazy_catch);
-  SCM_REDEFER_INTS;
-  scm_dynwinds = scm_acons (tag, handler, scm_dynwinds);
-  SCM_REALLOW_INTS;
-  answer = scm_apply (thunk, SCM_EOL, SCM_EOL);
-  SCM_REDEFER_INTS;
-  scm_dynwinds = SCM_CDR (scm_dynwinds);
-  SCM_REALLOW_INTS;
-  return answer;
+
+  c.tag = tag;
+  c.body_proc = thunk;
+
+  /* scm_internal_lazy_catch takes care of all the mechanics of
+     setting up a lazy catch tag; we tell it to call scm_body_thunk to
+     run the body, and scm_handle_by_proc to deal with any throws to
+     this catch.  The former receives a pointer to c, telling it how
+     to behave.  The latter receives a pointer to HANDLER, so it knows
+     who to call.  */
+  return scm_internal_lazy_catch (tag,
+				  scm_body_thunk, &c, 
+				  scm_handle_by_proc, &handler);
 }
 
-/* The user has thrown to an uncaught key --- print a message and die.
-   At boot time, we establish a catch-all that uses this as its handler.
-   1) If the user wants something different, they can use (catch #t
-   ...) to do what they like.
-   2) Outside the context of a read-eval-print loop, there isn't
-   anything else good to do; libguile should not assume the existence
-   of a read-eval-print loop.
-   3) Given that we shouldn't do anything complex, it's much more
-   robust to do it in C code.
 
-   HANDLER_DATA, if non-zero, is assumed to be a char * pointing to a
-   message header to print; if zero, we use "guile" instead.  That
-   text is followed by a colon, then the message described by ARGS.  */
+
+/* throwing */
 
+SCM_PROC(s_throw, "throw", 1, 0, 1, scm_throw);
 SCM
-scm_handle_by_message (handler_data, tag, args)
-     void *handler_data;
-     SCM tag;
+scm_throw (key, args)
+     SCM key;
      SCM args;
 {
-  char *prog_name = (char *) handler_data;
-  SCM p = scm_def_errp;
-
-  if (! prog_name)
-    prog_name = "guile";
-
-  scm_gen_puts (scm_regular_string, prog_name, p);
-  scm_gen_puts (scm_regular_string, ": ", p);
-
-  if (scm_ilength (args) >= 3)
-    {
-      SCM message = SCM_CADR (args);
-      SCM parts = SCM_CADDR (args);
-
-      scm_display_error_message (message, parts, p);
-    }
-  else
-    {
-      scm_gen_puts (scm_regular_string, "uncaught throw to ", p);
-      scm_prin1 (tag, p, 0);
-      scm_gen_puts (scm_regular_string, ": ", p);
-      scm_prin1 (args, p, 1);
-      scm_gen_putc ('\n', p);
-    }
-
-  exit (2);
+  /* May return if handled by lazy catch. */
+  return scm_ithrow (key, args, 1);
 }
 
 
-static char s_throw[];
 SCM
 scm_ithrow (key, args, noreturn)
      SCM key;
@@ -425,6 +589,7 @@ scm_ithrow (key, args, noreturn)
       if (winds == SCM_EOL)
 	abort ();
 
+      /* If the wind list is malformed, bail.  */
       if (SCM_IMP (winds) || SCM_NCONSP (winds))
 	abort ();
       
@@ -449,31 +614,45 @@ scm_ithrow (key, args, noreturn)
        SCM_CDAR (wind_goal) != jmpbuf;
        wind_goal = SCM_CDR (wind_goal))
     ;
-  if (!SCM_JMPBUFP (jmpbuf))
+
+  /* Is a lazy catch?  In wind list entries for lazy catches, the key
+     is bound to a lazy_catch smob, not a jmpbuf.  */
+  if (SCM_LAZY_CATCH_P (jmpbuf))
     {
+      struct lazy_catch *c = (struct lazy_catch *) SCM_CDR (jmpbuf);
       SCM oldwinds = scm_dynwinds;
       SCM handle, answer;
-      scm_dowinds (wind_goal, scm_ilength (scm_dynwinds) - scm_ilength (wind_goal));
+      scm_dowinds (wind_goal, (scm_ilength (scm_dynwinds)
+			       - scm_ilength (wind_goal)));
       SCM_REDEFER_INTS;
       handle = scm_dynwinds;
       scm_dynwinds = SCM_CDR (scm_dynwinds);
       SCM_REALLOW_INTS;
-      answer = scm_apply (jmpbuf, scm_cons (key, args), SCM_EOL);
+      answer = (c->handler) (c->handler_data, key, args);
       SCM_REDEFER_INTS;
       SCM_SETCDR (handle, scm_dynwinds);
       scm_dynwinds = handle;
       SCM_REALLOW_INTS;
-      scm_dowinds (oldwinds, scm_ilength (scm_dynwinds) - scm_ilength (oldwinds));
+      scm_dowinds (oldwinds, (scm_ilength (scm_dynwinds)
+			      - scm_ilength (oldwinds)));
       return answer;
     }
-  else
+
+  /* Otherwise, it's a normal catch.  */
+  else if (SCM_JMPBUFP (jmpbuf))
     {
       struct jmp_buf_and_retval * jbr;
-      scm_dowinds (wind_goal, scm_ilength (scm_dynwinds) - scm_ilength (wind_goal));
+      scm_dowinds (wind_goal, (scm_ilength (scm_dynwinds)
+			       - scm_ilength (wind_goal)));
       jbr = (struct jmp_buf_and_retval *)JBJMPBUF (jmpbuf);
       jbr->throw_tag = key;
       jbr->retval = args;
     }
+
+  /* Otherwise, it's some random piece of junk.  */
+  else
+    abort ();
+
 #ifdef DEBUG_EXTENSIONS
   scm_last_debug_frame = SCM_JBDFRAME (jmpbuf);
 #endif
@@ -481,20 +660,10 @@ scm_ithrow (key, args, noreturn)
 }
 
 
-SCM_PROC(s_throw, "throw", 1, 0, 1, scm_throw);
-SCM
-scm_throw (key, args)
-     SCM key;
-     SCM args;
-{
-  /* May return if handled by lazy catch. */
-  return scm_ithrow (key, args, 1);
-}
-
-
 void
 scm_init_throw ()
 {
   scm_tc16_jmpbuffer = scm_newsmob (&jbsmob);
+  tc16_lazy_catch = scm_newsmob (&lazy_catch_funs);
 #include "throw.x"
 }
