@@ -58,6 +58,12 @@ extern SCM scm_internal_catch SCM_P ((SCM tag,
 				      scm_catch_handler_t handler,
 				      void *handler_data));
 
+extern SCM scm_internal_lazy_catch SCM_P ((SCM tag,
+					   scm_catch_body_t body,
+					   void *body_data,
+					   scm_catch_handler_t handler,
+					   void *handler_data));
+
 /* The first argument to scm_body_thunk should be a pointer to one of
    these.  See the implementation of catch in throw.c.  */
 struct scm_body_thunk_data
@@ -77,10 +83,12 @@ extern SCM scm_body_thunk SCM_P ((void *, SCM));
 
 extern SCM scm_handle_by_proc SCM_P ((void *, SCM, SCM));
 extern SCM scm_handle_by_message SCM_P ((void *, SCM, SCM));
+extern int scm_exit_status SCM_P ((SCM args));
 
 extern SCM scm_catch SCM_P ((SCM tag, SCM thunk, SCM handler));
 extern SCM scm_lazy_catch SCM_P ((SCM tag, SCM thunk, SCM handler));
 extern SCM scm_ithrow SCM_P ((SCM key, SCM args, int noreturn));
+
 extern SCM scm_throw SCM_P ((SCM key, SCM args));
 extern void scm_init_throw SCM_P ((void));
 #endif  /* THROWH */
